@@ -2,8 +2,8 @@
 // Name:        BurnDlg.h
 // Purpose:     Burn dialog
 // Author:      Alex Thuering
-// Created:	23.03.2003
-// RCS-ID:      $Id: BurnDlg.cpp,v 1.27 2010/04/05 16:22:43 ntalex Exp $
+// Created:     23.03.2003
+// RCS-ID:      $Id: BurnDlg.cpp,v 1.32 2011/02/13 16:54:50 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -16,6 +16,19 @@
 #include <wx/progdlg.h>
 #include <wxVillaLib/utils.h>
 
+#ifdef HAVE_LIBUDEV
+extern "C" {
+#include <libudev.h>
+}
+#elif defined(__WXMSW__)
+#include <initguid.h>
+#include <devguid.h>
+#include <setupapi.h>
+#include <ddk/ntdddisk.h>
+#include <ddk/ntddscsi.h>
+#include <ddk/cfgmgr32.h>
+#endif
+
 enum {
 	TEMP_DIR_BT_ID = 2000,
 	TEMP_DIR_FIELD_ID,
@@ -45,9 +58,9 @@ END_EVENT_TABLE()
 
 BurnDlg::BurnDlg(wxWindow* parent, DVD* dvd, Cache* cache): wxDialog(parent, -1, wxEmptyString,
 		wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER) {
-    m_dvd = dvd;
-    m_cache = cache;
-    
+	m_dvd = dvd;
+	m_cache = cache;
+	
     // begin wxGlade: BurnDlg::BurnDlg
     m_tempDirLabel = new wxStaticText(this, wxID_ANY, _("Temp directory:"));
     m_tempDirText = new wxTextCtrl(this, TEMP_DIR_FIELD_ID, wxEmptyString);
@@ -70,7 +83,12 @@ BurnDlg::BurnDlg(wxWindow* parent, DVD* dvd, Cache* cache): wxDialog(parent, -1,
     m_formatCheck = new wxCheckBox(this, wxID_ANY, _("format DVD-RW"));
     m_deviceLabel = new wxStaticText(this, wxID_ANY, _("Device:"));
     const wxString *m_deviceChoice_choices = NULL;
+#if defined(HAVE_LIBUDEV) || defined(__WXMSW__) || defined(__WXMAC__)
+    m_deviceChoice = new wxComboBox(this, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, 0, m_deviceChoice_choices,
+    		wxCB_DROPDOWN|wxCB_READONLY);
+#else
     m_deviceChoice = new wxComboBox(this, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, 0, m_deviceChoice_choices, wxCB_DROPDOWN);
+#endif
     m_speedLabel = new wxStaticText(this, wxID_ANY, _("Speed:"));
     const wxString *m_speedChoice_choices = NULL;
     m_speedChoice = new wxComboBox(this, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, 0, m_speedChoice_choices, wxCB_DROPDOWN);
@@ -86,6 +104,230 @@ BurnDlg::BurnDlg(wxWindow* parent, DVD* dvd, Cache* cache): wxDialog(parent, -1,
     // end wxGlade
 }
 
+#ifdef HAVE_LIBUDEV
+void EnumDevices(wxArrayString& devices, wxComboBox* deviceChoice) {
+	udev* udev = udev_new();
+	if (!udev) {
+		wxLogError(wxT("Can't create udev"));
+	} else {
+		udev_enumerate* enumerate = udev_enumerate_new(udev);
+		udev_enumerate_add_match_property(enumerate, "ID_CDROM_DVD", "1");
+		udev_enumerate_scan_devices(enumerate);
+		udev_list_entry* devs = udev_enumerate_get_list_entry(enumerate);
+		udev_list_entry* dev_list_entry;
+		udev_list_entry_foreach(dev_list_entry, devs) {
+			udev_device* dev = udev_device_new_from_syspath(udev, udev_list_entry_get_name(dev_list_entry));
+			wxString devNode(udev_device_get_devnode(dev), *wxConvFileName);
+			wxString devModel(udev_device_get_property_value(dev, "ID_MODEL"), wxConvISO8859_1);
+			devices.Add(devNode);
+			deviceChoice->Append(devModel);
+		}
+	}
+}
+#elif defined(__WXMSW__)
+const int BUFFER_SIZE = 512;
+
+wxString GetDriveString(WCHAR* deviceId) {
+	wxString result;
+	int nLength = wcslen(deviceId);
+	deviceId[nLength] = '\\';
+	deviceId[nLength + 1] = 0;
+	// get volume mount point for the device path.
+	WCHAR volume[BUFFER_SIZE];
+	if (GetVolumeNameForVolumeMountPoint(deviceId, volume, BUFFER_SIZE)) {
+		TCHAR bufd[BUFFER_SIZE]; 
+		TCHAR drive[] = TEXT("C:\\");
+		for (TCHAR c = TEXT('C'); c < TEXT('Z');  c++) {
+			drive[0] = c;
+			if (GetVolumeNameForVolumeMountPoint(drive, bufd, BUFFER_SIZE) && !wcscmp(bufd, volume)) {
+				result = wxString(drive, wxConvISO8859_1).Mid(0, 2);
+				break;
+			}
+		}
+	}
+	deviceId[nLength] = 0;
+	return result;
+}
+
+typedef struct _SCSI_PASS_THROUGH_WITH_BUFFERS {
+	SCSI_PASS_THROUGH Spt;
+	ULONG Filler; // realign buffers to double word boundary
+	UCHAR SenseBuf[32];
+	UCHAR DataBuf[512];
+} SCSI_PASS_THROUGH_WITH_BUFFERS, *PSCSI_PASS_THROUGH_WITH_BUFFERS;
+
+#define SCSIOP_MODE_SENSE 0x1A
+#define MODE_PAGE_CAPABILITIES 0x2A
+
+bool IsDvdWriter(HANDLE hDevice) {
+	SCSI_PASS_THROUGH_WITH_BUFFERS sptwb;
+	ZeroMemory(&sptwb,sizeof(SCSI_PASS_THROUGH_WITH_BUFFERS));
+	sptwb.Spt.Length = sizeof (SCSI_PASS_THROUGH);
+	sptwb.Spt.PathId = 0;
+	sptwb.Spt.TargetId = 1;
+	sptwb.Spt.Lun = 0;
+	sptwb.Spt.CdbLength = 6;
+	sptwb.Spt.SenseInfoLength = 24;
+	sptwb.Spt.DataIn = SCSI_IOCTL_DATA_IN;
+	sptwb.Spt.DataTransferLength = 192;
+	sptwb.Spt.TimeOutValue = 2;
+	sptwb.Spt.DataBufferOffset = offsetof(SCSI_PASS_THROUGH_WITH_BUFFERS, DataBuf);
+	sptwb.Spt.SenseInfoOffset = offsetof(SCSI_PASS_THROUGH_WITH_BUFFERS, SenseBuf);
+	sptwb.Spt.Cdb[0] = SCSIOP_MODE_SENSE;
+	sptwb.Spt.Cdb[1] = 0x08;
+	sptwb.Spt.Cdb[2] = MODE_PAGE_CAPABILITIES;
+	sptwb.Spt.Cdb[4] = 192;
+	ULONG length = offsetof(SCSI_PASS_THROUGH_WITH_BUFFERS, DataBuf) + sptwb.Spt.DataTransferLength;
+	ULONG returned = 0;
+	bool status = DeviceIoControl(hDevice, IOCTL_SCSI_PASS_THROUGH, &sptwb,
+			sizeof(SCSI_PASS_THROUGH), &sptwb, length, &returned, false);
+	if (status && sptwb.Spt.ScsiStatus == 0) {
+		return (sptwb.DataBuf[7] & 0x10) || (sptwb.DataBuf[7] & 0x20);
+	}
+	return true; // cannot check
+}
+
+wxString GetProductId(HANDLE hDevice) {
+	STORAGE_PROPERTY_QUERY query;
+	query.PropertyId = StorageAdapterProperty;
+	query.QueryType = PropertyStandardQuery;
+	UCHAR outBuf[512];
+	ULONG returnedLength = 0;
+	bool status = DeviceIoControl(hDevice, IOCTL_STORAGE_QUERY_PROPERTY, &query, sizeof(STORAGE_PROPERTY_QUERY),
+			&outBuf, 512, &returnedLength, NULL);
+	if (!status)
+		return wxT("");
+	query.PropertyId = StorageDeviceProperty;
+	query.QueryType = PropertyStandardQuery;
+	status = DeviceIoControl(hDevice, IOCTL_STORAGE_QUERY_PROPERTY, &query, sizeof(STORAGE_PROPERTY_QUERY),
+			&outBuf, 512, &returnedLength, NULL);
+	if (!status)
+		return wxT("");
+	PSTORAGE_DEVICE_DESCRIPTOR  devDesc = (PSTORAGE_DEVICE_DESCRIPTOR) outBuf;
+	PUCHAR p = (PUCHAR) outBuf;
+	if (devDesc->ProductIdOffset && p[devDesc->ProductIdOffset])
+		return wxString((char*) (p + devDesc->ProductIdOffset), wxConvISO8859_1);
+	return wxT("");
+}
+
+bool GetDeviceHandle(HDEVINFO IntDevInfo, int index, HANDLE& hDevice, wxString& drive) {
+	hDevice = INVALID_HANDLE_VALUE;
+
+	SP_DEVICE_INTERFACE_DATA interfaceData;
+	interfaceData.cbSize = sizeof(SP_INTERFACE_DEVICE_DATA);
+	if (!SetupDiEnumDeviceInterfaces(IntDevInfo, 0, (LPGUID)(&GUID_DEVINTERFACE_CDROM), index, &interfaceData))
+		return false;
+	
+	DWORD reqSize;
+	if (!SetupDiGetDeviceInterfaceDetail(IntDevInfo, &interfaceData, NULL, 0, &reqSize, NULL)
+			&& GetLastError() != ERROR_INSUFFICIENT_BUFFER)
+		return false;
+	
+	DWORD iddSize = reqSize;
+	PSP_DEVICE_INTERFACE_DETAIL_DATA interfaceDetailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA) malloc(iddSize);
+	if (interfaceDetailData == NULL)
+		return false;
+	
+	bool result = true;
+	interfaceDetailData->cbSize = sizeof (SP_INTERFACE_DEVICE_DETAIL_DATA);
+	if (SetupDiGetDeviceInterfaceDetail(IntDevInfo, &interfaceData, interfaceDetailData, iddSize, &reqSize, NULL)) {
+		drive = GetDriveString(interfaceDetailData->DevicePath);
+		if (drive.length() > 0)
+			hDevice = CreateFile(interfaceDetailData->DevicePath, GENERIC_READ|GENERIC_WRITE,
+					FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, NULL);
+	} else
+		result = false;
+	
+	free(interfaceDetailData);
+	return result;
+}
+
+void EnumDevices(wxArrayString& devices, wxComboBox* deviceChoice) {
+	HDEVINFO hDevInfo = SetupDiGetClassDevs((LPGUID) &GUID_DEVCLASS_CDROM, NULL, NULL, DIGCF_PRESENT);
+	if (hDevInfo == INVALID_HANDLE_VALUE) {
+		wxLogError(wxT("SetupDiGetClassDevs(GUID_DEVCLASS_CDROM) failed with error: %d"), GetLastError());
+		return;
+	}
+
+	HDEVINFO hIntDevInfo = SetupDiGetClassDevs((LPGUID) &GUID_DEVINTERFACE_CDROM, NULL, NULL,
+			(DIGCF_PRESENT | DIGCF_INTERFACEDEVICE));
+	if (hIntDevInfo == INVALID_HANDLE_VALUE) {
+		wxLogError(wxT("SetupDiGetClassDevs(GUID_DEVINTERFACE_CDROM) failed with error: %d"), GetLastError());
+		return;
+	}
+
+	// enumerate all the DVD devices
+	int index = 0;
+	while (true) {
+		HANDLE hDevice;
+		wxString drive;
+		if (!GetDeviceHandle(hIntDevInfo, index++, hDevice, drive))
+			break;
+		if (drive.length() == 0)
+			continue;
+		
+		if (hDevice != INVALID_HANDLE_VALUE) {
+			wxString productId = GetProductId(hDevice);
+			if (IsDvdWriter(hDevice)) {
+				devices.Add(drive);
+				deviceChoice->Append(drive + wxT(" ") + productId);
+			}
+			CloseHandle(hDevice);
+		} else {
+			devices.Add(drive);
+			deviceChoice->Append(drive);
+		}
+	}
+	SetupDiDestroyDeviceInfoList(hDevInfo);
+	SetupDiDestroyDeviceInfoList(hIntDevInfo);
+}
+#elif defined(__WXMAC__)
+extern "C" void EnumDevices(wxArrayString& devices, wxComboBox* deviceChoice);
+#else
+void EnumDevices(wxArrayString& devices, wxComboBox* deviceChoice) {
+	wxArrayString allDevices;
+	wxDir d(wxT("/dev"));
+	wxString fname;
+	if (d.GetFirst(&fname, wxT("dvd*"), wxDIR_FILES))
+		do { allDevices.Add(wxT("/dev/") + fname); } while (d.GetNext(&fname));
+	if (d.GetFirst(&fname, wxT("cdrom*"), wxDIR_FILES))
+		do { allDevices.Add(wxT("/dev/") + fname); } while (d.GetNext(&fname));
+	if (d.GetFirst(&fname, wxT("sg*"), wxDIR_FILES))
+		do { allDevices.Add(wxT("/dev/") + fname); } while (d.GetNext(&fname));
+	allDevices.Sort();
+	// get device name
+	wxString scanCmd = s_config.GetBurnScanCmd();
+	if (scanCmd.length()) {
+		wxProgressDialog* pdlg = new wxProgressDialog(_("Scan devices"), _("Please wait..."),
+				allDevices.GetCount() - 1, deviceChoice->GetParent(), wxPD_AUTO_HIDE | wxPD_APP_MODAL | wxPD_CAN_ABORT);
+		pdlg->Show();
+		pdlg->SetFocus();
+		for (unsigned int i = 0; i < allDevices.GetCount(); i++) {
+			wxString cmd = scanCmd;
+			cmd.Replace(wxT("$DEVICE"), allDevices[i]);
+			wxArrayString output;
+			wxExecute(cmd, output, wxEXEC_SYNC | wxEXEC_NODISABLE);
+			if (output.Count() > 0 && output[0].length() > 7 && output[0].SubString(0, 7) == wxT("INQUIRY:")) {
+				wxString dev = output[0].AfterFirst(wxT('[')).BeforeLast(wxT(']'));
+				dev = allDevices[i] + wxT(" ") + dev.BeforeFirst(wxT(']')).Trim() + wxT(" ")
+						+ dev.AfterFirst(wxT('[')).BeforeFirst(wxT(']')).Trim();
+				devices.Add(allDevices[i]);
+				deviceChoice->Append(dev);
+			}
+			if (!pdlg->Update(i))
+				break;
+		}
+		pdlg->Hide();
+	}
+	if (devices.Count() == 0) {
+		for (unsigned int i=0; i<allDevices.GetCount(); i++) {
+			devices.Add(allDevices[i]);
+			deviceChoice->Append(allDevices[i]);
+		}
+	}
+}
+#endif //!HAVE_LIBUDEV && !__WXMSW__
+
 void BurnDlg::set_properties() {
     // begin wxGlade: BurnDlg::set_properties
     SetTitle(_("Burn"));
@@ -94,63 +336,15 @@ void BurnDlg::set_properties() {
     m_startBt->SetDefault();
     // end wxGlade
     
-    m_tempDirText->SetFocus();
+	m_tempDirText->SetFocus();
 	
 	int h = m_tempDirText->GetSize().GetHeight() > 21 ? m_tempDirText->GetSize().GetHeight() : 21;
 	m_tempDirBt->SetSizeHints(h, h, h, h);
 	m_isoBt->SetSizeHints(h, h, h, h);
 	
 	// set devices
-	wxArrayString devices;
-#ifdef __WXMSW__
-	for (int i=0; i<26; i++) {
-        char buf[] = { i+64, 0 };
-        wxString drive = wxString(buf, wxConvISO8859_1) + wxT(":");
-        if (GetDriveType(drive) == DRIVE_CDROM)
-            devices.Add(drive);
-	}
-#else
-    wxDir d(wxT("/dev"));
-    wxString fname;
-    if (d.GetFirst(&fname, wxT("dvd*"), wxDIR_FILES))
-    	do { devices.Add(wxT("/dev/") + fname); } while (d.GetNext(&fname));
-    if (d.GetFirst(&fname, wxT("cdrom*"), wxDIR_FILES))
-    	do { devices.Add(wxT("/dev/") + fname); } while (d.GetNext(&fname));
-    if (d.GetFirst(&fname, wxT("sg*"), wxDIR_FILES))
-    	do { devices.Add(wxT("/dev/") + fname); } while (d.GetNext(&fname));
-    devices.Sort();
-#endif
-    wxString scanCmd = s_config.GetBurnScanCmd();
-    if (scanCmd.length()) {
-		wxProgressDialog* pdlg = new wxProgressDialog(_("Scan devices"), _("Please wait..."),
-				devices.GetCount() - 1, GetParent(), wxPD_AUTO_HIDE | wxPD_APP_MODAL | wxPD_CAN_ABORT);
-		pdlg->Show();
-		pdlg->SetFocus();
-		for (unsigned int i = 0; i < devices.GetCount(); i++) {
-			wxString cmd = scanCmd;
-			cmd.Replace(wxT("$DEVICE"), devices[i]);
-#ifdef __WXMSW__
-			cmd = wxGetAppPath() + wxString(wxFILE_SEP_PATH) + cmd;
-#endif
-			wxArrayString output;
-			wxExecute(cmd, output, wxEXEC_SYNC | wxEXEC_NODISABLE);
-			if (output.Count() > 0 && output[0].length() > 7
-					&& output[0].SubString(0, 7) == wxT("INQUIRY:")) {
-				wxString dev = output[0].AfterFirst(wxT('[')).BeforeLast(wxT(']'));
-				dev = devices[i] + wxT(" ") + dev.BeforeFirst(wxT(']')).Trim() + wxT(" ")
-						+ dev.AfterFirst(wxT('[')).BeforeFirst(wxT(']')).Trim();
-				m_devices.Add(devices[i]);
-				m_deviceChoice->Append(dev);
-			}
-			if (!pdlg->Update(i))
-				break;
-		}
-		pdlg->Hide();
-	}
-    if (m_devices.Count() == 0)
-        for (unsigned int i=0; i<devices.GetCount(); i++)
-            m_deviceChoice->Append(devices[i]);
-    
+	EnumDevices(m_devices, m_deviceChoice);
+	
 	m_speedChoice->Append(_T("auto"));
 	for (int speed = 1; speed <= 8; speed = speed * 2)
 		m_speedChoice->Append(wxString::Format(_T("%dx"), speed));
@@ -221,12 +415,12 @@ void BurnDlg::do_layout() {
     panelSizer->Add(formatSizer, 1, wxEXPAND, 0);
     deviceSizer->Add(16, 8, 0, 0, 0);
     deviceSizer->Add(m_deviceLabel, 0, wxRIGHT|wxALIGN_CENTER_VERTICAL, 4);
-    deviceSizer->Add(m_deviceChoice, 0, 0, 0);
-    deviceSizer->Add(12, 20, 0, 0, 0);
+    deviceSizer->Add(m_deviceChoice, 0, wxALIGN_CENTER_VERTICAL, 0);
+    deviceSizer->Add(12, 8, 0, 0, 0);
     deviceSizer->Add(m_speedLabel, 0, wxRIGHT|wxALIGN_CENTER_VERTICAL, 12);
-    deviceSizer->Add(m_speedChoice, 0, 0, 0);
+    deviceSizer->Add(m_speedChoice, 0, wxALIGN_CENTER_VERTICAL, 0);
     deviceSizer->Add(m_devPanel, 1, wxEXPAND, 0);
-    panelSizer->Add(deviceSizer, 1, wxTOP|wxBOTTOM|wxEXPAND, 4);
+    panelSizer->Add(deviceSizer, 1, wxBOTTOM|wxEXPAND, 4);
     panelSizer->Add(m_addECCCheck, 0, 0, 0);
     mainSizer->Add(panelSizer, 0, wxALL|wxEXPAND, 6);
     mainSizer->Add(m_spacer, 1, wxEXPAND, 0);
@@ -255,10 +449,15 @@ wxString BurnDlg::GetDevice() {
 void BurnDlg::SetDevice(wxString device) {
 	if (m_devices.Count()) {
 		int n = device.length() > 0 ? m_devices.Index(device) : 0;
-		if (n >= 0)
+		if (n >= 0) {
 			m_deviceChoice->SetSelection(n);
-		else
+		} else {
+#ifdef HAVE_LIBUDEV
+			m_deviceChoice->SetSelection(0);
+#else
 			m_deviceChoice->SetValue(device);
+#endif
+		}
 	} else
 		m_deviceChoice->SetValue(device);
 }
@@ -289,6 +488,16 @@ void BurnDlg::UpdateCtrls(bool def) {
 	m_formatCheck->SetValue(s_config.GetFormatDo(def));
 	SetDevice(s_config.GetBurnDevice(def));
 	SetSpeed(s_config.GetBurnSpeed(def));
+#if defined(__WXMSW__) || defined(__WXMAC__)
+	bool hasDevices = m_deviceChoice->GetCount() > 0;
+#else
+	bool hasDevices = true;
+#endif
+	if (!hasDevices) {
+ 		if (m_burnRadioBt->GetValue())
+ 			m_isoRadioBt->SetValue(true);
+ 		m_burnRadioBt->Enable(false);
+	}
 	wxCommandEvent evt;
 	OnRadioBt(evt);
 }
@@ -425,7 +634,7 @@ void BurnDlg::OnOk(wxCommandEvent& event) {
 		return;
 	}
 	wxString tmpDir = m_tempDirText->GetValue();
-	if (m_cache->GetTempDir() != tmpDir && m_cache->GetCount() > 0
+	if (m_cache->IsInitialized() && m_cache->GetTempDir() != tmpDir && m_cache->GetCount() > 0
 			&& wxMessageBox(wxString::Format(
 					_("There are %d transcoded file(s) in the cache. They\nwill be removed if you change temporary directory."), m_cache->GetCount()),
 					_("Burn"), wxOK|wxCANCEL|wxICON_INFORMATION, this) == wxCANCEL)
