@@ -3,7 +3,7 @@
 // Purpose:     Configuration
 // Author:      Alex Thuering
 // Created:     27.03.2003
-// RCS-ID:      $Id: Config.h,v 1.61 2010/04/03 21:47:01 ntalex Exp $
+// RCS-ID:      $Id: Config.h,v 1.66 2010/09/16 18:29:26 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -28,10 +28,11 @@ const int DEF_SPECT_RATIO		= 1;
 const int DEF_CHAPTER_LENGTH	= 10;
 const wxString DEF_BUTTON		= wxT("frame.xml");
 const wxString DEF_TEXT_BUTTON	= wxT("text-v2.xml");
+const bool DEF_ACCEPT_INVALID_ACTIONS = false;
 const wxString DEF_FILE_BROWSER_DIR = wxT("");
-const wxString DEF_FILE_BROWSER_LAST_DIR = wxGetHomeDir();
-const wxString DEF_LAST_PROJECT_DIR  = wxGetHomeDir();
-const wxString DEF_LAST_ADD_DIR = wxGetHomeDir();
+#define DEF_FILE_BROWSER_LAST_DIR wxGetHomeDir()
+#define DEF_LAST_PROJECT_DIR wxGetHomeDir()
+#define DEF_LAST_ADD_DIR wxGetHomeDir()
 const int DEF_UNDO_HISTORY_SIZE = 100;
 const bool DEF_REMOVE_TEMP = true;
 const bool DEF_VBR = false;
@@ -39,6 +40,7 @@ const int DEF_VIDEO_BITRATE = -1;
 const int DEF_AUDIO_BITRATE = 224;
 const int DEF_MENU_FRAME_COUNT = 100;
 const int DEF_MENU_VIDEO_BITRATE = 6000;
+const bool DEF_DRAW_BUTTONS_ON_BACKGROUND = true;
 const int DEF_SLIDESHOW_VIDEO_BITRATE = 6000;
 const int DEF_THREAD_COUNT = wxThread::GetCPUCount() > 0 ? wxThread::GetCPUCount() : 1;
 const int DEF_DVD_RESERVED_SPACE = 80000; // KB
@@ -48,14 +50,19 @@ const int DEF_DVD_RESERVED_SPACE = 80000; // KB
 	const wxString DEF_PREVIEW_CMD   = _T("wmplayer");
 	const wxString DEF_BURN_DEVICE   = _T("");
 	const wxString DEF_ADD_ECC_CMD   = _T("\"C:\\Program Files\\dvdisaster\\dvdisaster\" -mRS02 -n dvd -c -i \"$FILE\"");
+#else
+#ifdef __WXMAC__
+#define DEF_GEN_TEMP_DIR (wxGetHomeDir() + wxT("/Temp"))
 #else
 	const wxString DEF_GEN_TEMP_DIR = wxT("/tmp");
+#endif
 	const bool DEF_PREVIEW_DO        = true;
 	const wxString DEF_PREVIEW_CMD   = _T("xine \"dvd:/$DIR\"");
 	const wxString DEF_BURN_DEVICE   = _T("/dev/dvd");
 	const wxString DEF_ADD_ECC_CMD   = _T("dvdisaster -mRS02 -n dvd -c -i \"$FILE\"");
 #endif
-const bool DEF_USE_MPLEX		 = true;
+const bool DEF_USE_MPLEX = false;
+const bool DEF_USE_MPLEX_FOR_MENUS = true;
 const wxString DEF_MPLEX_CMD     = _T("mplex -f 8 -S 0 -M -V -o \"$FILE_OUT\" \"$FILE_VIDEO\" \"$FILE_AUDIO\"");
 const wxString DEF_SPUMUX_CMD    = _T("spumux -s $STREAM \"$FILE_CONF\"");
 const wxString DEF_DVDAUTHOR_CMD = _T("dvdauthor -o \"$DIR\" -x \"$FILE_CONF\"");
@@ -63,14 +70,19 @@ const wxString DEF_ISO_CMD       = _T("mkisofs -V \"$VOL_ID\" -o \"$FILE\" -dvd-
 const wxString DEF_ISO_SIZE_CMD  = _T("mkisofs -quiet -print-size \"$DIR\"");
 const wxString DEF_BURN_SCAN_CMD = _T("dvd+rw-mediainfo $DEVICE");
 const wxString DEF_BURN_CMD      = _T("growisofs -V \"$VOL_ID\" -dvd-compat -Z $DEV -dvd-video \"$DIR\" $SPEEDSTR");
+const wxString DEF_BURN_SPEED_OPT= _T("-speed=$SPEED");
+#ifdef __WXMAC__
+const wxString DEF_BURN_ISO_CMD  = _T("/usr/bin/hdiutil burn -device $DEV $SPEEDSTR \"$FILE\"");
+const wxString DEF_FORMAT_CMD    = _T("/usr/bin/hdiutil burn -device $DEV -erase");
+#else
 const wxString DEF_BURN_ISO_CMD  = _T("growisofs -dvd-compat -Z $DEV=\"$FILE\" -use-the-force-luke=dao:$SIZE $SPEEDSTR");
+const wxString DEF_FORMAT_CMD    = _T("dvd+rw-format -force $DEV");
+#endif
 const bool DEF_GENERATE_DO       = false;
 const bool DEF_ISO_DO            = false;
 const bool DEF_BURN_DO           = true;
-const wxString DEF_BURN_SPEED_OPT= _T("-speed=$SPEED");
 const bool DEF_FORMAT_DO         = false;
-const wxString DEF_FORMAT_CMD    = _T("dvd+rw-format -force $DEV");
-const wxString DEF_ISO_SAVETO = wxGetHomeDir() + wxFILE_SEP_PATH + _T("dvd.iso");
+#define DEF_ISO_SAVETO wxGetHomeDir() + wxFILE_SEP_PATH + _T("dvd.iso")
 const int DEF_BURN_SPEED      = 0;
 const bool DEF_ADD_ECC_DO        = false;
 const int DEF_KEY = 0;
@@ -117,6 +129,7 @@ public:
 	CONFIG_PROP_INT(DefChapterLength, _T("Interface/DefChapterLength"), DEF_CHAPTER_LENGTH)
 	CONFIG_PROP(DefButton, _T("Interface/DefButton"), DEF_BUTTON)
 	CONFIG_PROP(DefTextButton, _T("Interface/DefTextButton"), DEF_TEXT_BUTTON)
+	CONFIG_PROP_BOOL(AcceptInvalidActions, _T("Interface/AcceptInvalidActions"), DEF_ACCEPT_INVALID_ACTIONS)
 	CONFIG_PROP(FileBrowserDir, _T("Interface/FileBrowserDir"), DEF_FILE_BROWSER_DIR)
 	CONFIG_PROP(FileBrowserLastDir, _T("Interface/FileBrowserLastDir"), DEF_FILE_BROWSER_LAST_DIR)
 	CONFIG_PROP(LastProjectDir, _T("Interface/LastProjectDir"), DEF_LAST_PROJECT_DIR)
@@ -132,10 +145,12 @@ public:
 	CONFIG_PROP_INT(AudioBitrate, _T("Generate/AudioBitrate"), DEF_AUDIO_BITRATE)
 	CONFIG_PROP_INT(MenuFrameCount,_T("Generate/MenuFrameCount"), DEF_MENU_FRAME_COUNT)
 	CONFIG_PROP_INT(MenuVideoBitrate, _T("Generate/MenuVideoBitrate"), DEF_MENU_VIDEO_BITRATE)
+	CONFIG_PROP_BOOL(DrawButtonsOnBackground, _T("Generate/DrawButtonsOnBackground"), DEF_DRAW_BUTTONS_ON_BACKGROUND)
     CONFIG_PROP_INT(SlideshowVideoBitrate, _T("Generate/SlideshowVideoBitrate"), DEF_SLIDESHOW_VIDEO_BITRATE)
     CONFIG_PROP_INT(ThreadCount, _T("Generate/ThreadCount"), DEF_THREAD_COUNT)
     CONFIG_PROP_INT(DvdReservedSpace, _T("Generate/DvdReservedSpace"), DEF_DVD_RESERVED_SPACE)
     CONFIG_PROP_BOOL(UseMplex, _T("Generate/UseMplex"), DEF_USE_MPLEX)
+    CONFIG_PROP_BOOL(UseMplexForMenus, _T("Generate/UseMplexForMenus"), DEF_USE_MPLEX_FOR_MENUS)
     CONFIG_PROP(MplexCmd,   _T("Generate/MplexCmd"), DEF_MPLEX_CMD)
 	CONFIG_PROP(SpumuxCmd,  _T("Generate/SpumuxCmd"), DEF_SPUMUX_CMD)
 	CONFIG_PROP(DvdauthorCmd, _T("Generate/DvdauthorCmd"), DEF_DVDAUTHOR_CMD)
