@@ -2,8 +2,8 @@
 // Name:        DVD.cpp
 // Purpose:     The class to store a DVD Structure (Titles/Menus)
 // Author:      Alex Thuering
-// Created:	29.01.2003
-// RCS-ID:      $Id: DVD.cpp,v 1.95 2010/10/10 18:03:10 ntalex Exp $
+// Created:     29.01.2003
+// RCS-ID:      $Id: DVD.cpp,v 1.98 2011/03/10 20:01:27 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -75,10 +75,15 @@ bool Cell::PutXML(wxSvgXmlNode* node) {
 
 TextSub::TextSub(wxString filename) {
 	m_filename = filename;
-	m_characterSet = wxT("ISO-8859-1");
+	m_characterSet = wxT("UTF-8");
 	m_fontFamily = wxT("Arial");
 	m_fontStyle = wxT("Regular");
 	m_fontSize = 28;
+	m_fillColour = *wxWHITE;
+	m_outlineColour = *wxBLACK;
+    m_outlineThickness = 3.0;
+    m_shadowColour = *wxBLACK;
+    m_shadowOffset = wxPoint(0, 0);
 	m_alignment = (wxAlignment) (wxALIGN_CENTER_HORIZONTAL | wxALIGN_BOTTOM);
 	m_leftMargin = 60;
 	m_rightMargin = 60;
@@ -86,20 +91,28 @@ TextSub::TextSub(wxString filename) {
 	m_bottomMargin = 30;
 	m_subtitleFps = 25;
 	m_movieFps = 25;
-	m_movieWidth = 720;
-	m_movieHeight = 574;
+	m_movieSize = wxSize(720, 574);
 }
 
 wxString TextSub::GetFontFile() {
-	return SubtitlePropDlg::GetFontMap()[m_fontFamily][m_fontStyle];
+	map<wxString, wxString>& fonts = SubtitlePropDlg::GetFontMap()[m_fontFamily];
+	if (fonts.find(m_fontStyle) != fonts.end())
+		return fonts[m_fontStyle];
+	if (fonts.find(wxT("Normal")) != fonts.end())
+		return fonts[wxT("Normal")];
+	if (fonts.find(wxT("Standard")) != fonts.end())
+		return fonts[wxT("Standard")];
+	return fonts.begin() != fonts.end() ? fonts.begin()->second : wxT("");
 }
 
 wxSvgXmlNode* TextSub::GetXML(DVDFileType type) {
 	wxSvgXmlNode* textsubNode = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, _T("textsub"));
 	TextSub* defaults = new TextSub(wxT(""));
 	textsubNode->AddProperty(_T("filename"), m_filename);
-	if (m_characterSet != defaults->m_characterSet)
+	if (type == DVDAUTHOR_XML || m_characterSet != defaults->m_characterSet)
 		textsubNode->AddProperty(_T("characterset"), m_characterSet);
+	
+	// font
 	if (type == DVDAUTHOR_XML) {
 		if (GetFontFile().length() > 0)
 			textsubNode->AddProperty(wxT("font"), GetFontFile());
@@ -111,48 +124,101 @@ wxSvgXmlNode* TextSub::GetXML(DVDFileType type) {
 	}
 	if (m_fontSize != defaults->m_fontSize)
 		textsubNode->AddProperty(_T("fontsize"), wxString::Format(wxT("%g"), m_fontSize));
+	
+	// colours
+	if (m_fillColour.IsOk() && m_fillColour != defaults->m_fillColour)
+		textsubNode->AddProperty(wxT("fill-color"), wxString::Format(wxT("#%02x%02x%02x"),
+				m_fillColour.Red(), m_fillColour.Green(), m_fillColour.Blue()));
+	if (m_outlineColour.IsOk() && m_outlineColour != defaults->m_outlineColour)
+		textsubNode->AddProperty(wxT("outline-color"), wxString::Format(wxT("#%02x%02x%02x"),
+				m_outlineColour.Red(), m_outlineColour.Green(), m_outlineColour.Blue()));
+	if (m_outlineThickness != defaults->m_outlineThickness)
+		textsubNode->AddProperty(wxT("outline-thickness"), wxString::Format(wxT("%f"), m_outlineThickness));
+	if (m_shadowColour.IsOk() && m_shadowColour != defaults->m_shadowColour)
+		textsubNode->AddProperty(wxT("shadow-color"), wxString::Format(wxT("#%02x%02x%02x"),
+				m_shadowColour.Red(), m_shadowColour.Green(), m_shadowColour.Blue()));
+	if (m_shadowOffset != defaults->m_shadowOffset)
+		textsubNode->AddProperty(wxT("shadow-offset"),
+				wxString::Format(wxT("%d,%d"), m_shadowOffset.x, m_shadowOffset.y));
+	
+	// alignment
 	wxString hAlignment = m_alignment & wxALIGN_CENTER_HORIZONTAL ? wxT("center")
 			: m_alignment & wxALIGN_RIGHT ? wxT("right") : wxT("left");
 	if ((type == DVDAUTHOR_XML && hAlignment != wxT("left"))
 			|| (type == DVDSTYLER_XML && hAlignment != wxT("center")))
-		textsubNode->AddProperty(_T("horizontal-alignment"), hAlignment);
+		textsubNode->AddProperty(wxT("horizontal-alignment"), hAlignment);
 	wxString vAlignment = m_alignment & wxALIGN_CENTER_VERTICAL ? wxT("center")
 				: m_alignment & wxALIGN_BOTTOM ? wxT("bottom") : wxT("top");
 	if (vAlignment != wxT("bottom"))
-		textsubNode->AddProperty(_T("vertical-alignment"), vAlignment);
+		textsubNode->AddProperty(wxT("vertical-alignment"), vAlignment);
+	
+	// margin
 	if (m_leftMargin != defaults->m_leftMargin)
-		textsubNode->AddProperty(_T("left-margin"), wxString::Format(wxT("%d"), m_leftMargin));
+		textsubNode->AddProperty(wxT("left-margin"), wxString::Format(wxT("%d"), m_leftMargin));
 	if (m_rightMargin != defaults->m_rightMargin)
-		textsubNode->AddProperty(_T("right-margin"), wxString::Format(wxT("%d"), m_rightMargin));
+		textsubNode->AddProperty(wxT("right-margin"), wxString::Format(wxT("%d"), m_rightMargin));
 	if (m_topMargin != defaults->m_topMargin)
-		textsubNode->AddProperty(_T("top-margin"), wxString::Format(wxT("%d"), m_topMargin));
+		textsubNode->AddProperty(wxT("top-margin"), wxString::Format(wxT("%d"), m_topMargin));
 	if (m_bottomMargin != defaults->m_bottomMargin)
-		textsubNode->AddProperty(_T("bottom-margin"), wxString::Format(wxT("%d"), m_bottomMargin));
+		textsubNode->AddProperty(wxT("bottom-margin"), wxString::Format(wxT("%d"), m_bottomMargin));
+	
+	// fps
 	if (m_subtitleFps != defaults->m_subtitleFps)
-		textsubNode->AddProperty(_T("subtitle-fps"), wxString::Format(wxT("%g"), m_subtitleFps));
-	if (m_movieFps != defaults->m_movieFps)
-		textsubNode->AddProperty(_T("movie-fps"), wxString::Format(wxT("%g"), m_movieFps));
-	if (m_movieWidth != defaults->m_movieWidth)
-		textsubNode->AddProperty(_T("movie-width"), wxString::Format(wxT("%d"), m_movieWidth));
-	if (m_movieHeight != defaults->m_movieHeight)
-		textsubNode->AddProperty(_T("movie-height"), wxString::Format(wxT("%d"), m_movieHeight));
+		textsubNode->AddProperty(wxT("subtitle-fps"), wxString::Format(wxT("%g"), m_subtitleFps));
+	
+	// movie values
+	if (type == DVDAUTHOR_XML) {
+		if (m_movieFps != defaults->m_movieFps)
+			textsubNode->AddProperty(wxT("movie-fps"), wxString::Format(wxT("%g"), m_movieFps));
+		if (m_movieSize.x != defaults->m_movieSize.x)
+			textsubNode->AddProperty(wxT("movie-width"), wxString::Format(wxT("%d"), m_movieSize.x));
+		if (m_movieSize.y != defaults->m_movieSize.y)
+			textsubNode->AddProperty(wxT("movie-height"), wxString::Format(wxT("%d"), m_movieSize.y));
+		if (m_aspectRatio != arAUTO)
+			textsubNode->AddProperty(wxT("aspect"), m_aspectRatio == ar4_3 ? wxT("4:3") : wxT("16:9"));
+	}
 	delete defaults;
 	return textsubNode;
 }
 
+wxColour ParseColour(const wxString& value) {
+	long r = 0, g = 0, b = 0;
+	value.Mid(1, 2).ToLong(&r, 16);
+	value.Mid(3, 2).ToLong(&g, 16);
+	value.Mid(5, 2).ToLong(&b, 16);
+	return wxColour(r, g, b);
+}
+
 bool TextSub::PutXML(wxSvgXmlNode* node) {
 	wxString val;
 	long lval;
+	long lval2;
 	double dval;
 
 	if (!node->GetPropVal(wxT("filename"), &m_filename))
 		return false;
-
 	node->GetPropVal(wxT("characterset"), &m_characterSet);
+	
+	// font
 	node->GetPropVal(wxT("fontFamily"), &m_fontFamily);
 	node->GetPropVal(wxT("fontStyle"), &m_fontStyle);
 	if (node->GetPropVal(wxT("fontsize"), &val) && val.ToDouble(&dval))
 		m_fontSize = dval;
+	
+	// colours
+	if (node->GetPropVal(wxT("fill-color"), &val) && val.length() == 7 && val.GetChar(0) == wxT('#'))
+		m_fillColour = ParseColour(val);
+	if (node->GetPropVal(wxT("outline-color"), &val) && val.length() == 7 && val.GetChar(0) == wxT('#'))
+		m_outlineColour = ParseColour(val);
+	if (node->GetPropVal(wxT("outline-thickness"), &val) && val.ToDouble(&dval))
+		m_outlineThickness = dval;
+	if (node->GetPropVal(wxT("shadow-color"), &val) && val.length() == 7 && val.GetChar(0) == wxT('#'))
+		m_shadowColour = ParseColour(val);
+	if (node->GetPropVal(wxT("shadow-offset"), &val) && val.Find(wxT(',')) > 0
+			&& val.Mid(0, val.Find(wxT(','))).ToLong(&lval) && val.Mid(val.Find(wxT(',')) + 1).ToLong(&lval2))
+		m_shadowOffset = wxPoint(lval, lval2);
+	
+	// alignment
 	m_alignment = wxALIGN_CENTER_HORIZONTAL;
 	if (node->GetPropVal(wxT("horizontal-alignment"), &val)) {
 		if (val == wxT("left"))
@@ -169,6 +235,8 @@ bool TextSub::PutXML(wxSvgXmlNode* node) {
 			m_alignment = (wxAlignment) (m_alignment | wxALIGN_BOTTOM);
 	} else
 		m_alignment = (wxAlignment) (m_alignment | wxALIGN_BOTTOM);
+	
+	// margin
 	if (node->GetPropVal(wxT("left-margin"), &val) && val.ToLong(&lval))
 		m_leftMargin = lval;
 	if (node->GetPropVal(wxT("right-margin"), &val) && val.ToLong(&lval))
@@ -177,21 +245,18 @@ bool TextSub::PutXML(wxSvgXmlNode* node) {
 		m_topMargin = lval;
 	if (node->GetPropVal(wxT("bottom-margin"), &val) && val.ToLong(&lval))
 		m_bottomMargin = lval;
+	
+	// fps
 	if (node->GetPropVal(wxT("subtitle-fps"), &val) && val.ToDouble(&dval))
 		m_subtitleFps = dval;
-	if (node->GetPropVal(wxT("movie-fps"), &val) && val.ToDouble(&dval))
-		m_movieFps = dval;
-	if (node->GetPropVal(wxT("movie-width"), &val) && val.ToLong(&lval))
-		m_movieWidth = lval;
-	if (node->GetPropVal(wxT("movie-height"), &val) && val.ToLong(&lval))
-		m_movieHeight = lval;
 
 	return true;
 }
 
-bool TextSub::SaveSpumux(wxString filename) {
+bool TextSub::SaveSpumux(wxString filename, VideoFormat videoFormat) {
 	wxSvgXmlDocument xml;
 	wxSvgXmlNode* root = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("subpictures"));
+	root->AddProperty(wxT("format"), isNTSC(videoFormat) ? wxT("NTSC") : wxT("PAL"));
 	wxSvgXmlNode* streamNode = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("stream"));
 	streamNode->AddChild(GetXML(DVDAUTHOR_XML));
 	root->AddChild(streamNode);
@@ -241,10 +306,12 @@ wxString Stream::GetSourceFormat() {
 				result += wxT(" (PAL)");
 			else if (GetSourceVideoFormat() == vfNTSC)
 				result += wxT(" (NTSC)");
-			if (round(GetSourceAspectRatio()*100) == 133)
+			if (round(GetSourceAspectRatio()*100) == 133 || round(GetSourceAspectRatio()*100) == 136)
 				result += wxT(", 4:3");
-			else if (round(GetSourceAspectRatio()*100) == 155)
+			else if (round(GetSourceAspectRatio()*100) == 178)
 				result += wxT(", 16:9");
+			else
+				result += wxString::Format(wxT(", 1:%0.2f"), GetSourceAspectRatio());
 		}
 		break;
 	case stAUDIO:
@@ -288,11 +355,18 @@ Vob::Vob(Slideshow* slideshow) {
 
 void Vob::Init(Menu* menu, Slideshow* slideshow) {
 	m_pause = 0;
+	m_startTime = 0;
+	m_recordingTime = -1;
 	m_menu = menu;
 	m_slideshow = slideshow;
 	m_doNotTranscode = false;
 	m_interlaced = true;
 	m_firstField = ffAUTO;
+	m_keepAspectRatio = true;
+	for (int i = 0; i < 4; i++) {
+		m_pad.push_back(0);
+		m_crop.push_back(0);
+	}
 }
 
 Vob::Vob(const Vob& vob): m_pad(vob.m_pad), m_crop(vob.m_crop) {
@@ -306,11 +380,14 @@ Vob::Vob(const Vob& vob): m_pad(vob.m_pad), m_crop(vob.m_crop) {
 	m_menu = vob.m_menu;
 	m_slideshow = vob.m_slideshow;
 	m_duration = vob.m_duration;
+	m_startTime = vob.m_startTime;
+	m_recordingTime = vob.m_recordingTime;
 	for (unsigned int i = 0; i<vob.m_streams.size(); i++)
 		m_streams.Add(new Stream(*vob.m_streams[i]));
 	VECTOR_COPY(vob.m_cells, m_cells, Cell);
 	m_interlaced = vob.m_interlaced;
 	m_firstField = vob.m_firstField;
+	m_keepAspectRatio = vob.m_keepAspectRatio;
 	m_doNotTranscode = vob.m_doNotTranscode;
 }
 
@@ -341,8 +418,7 @@ bool Vob::SetFilename(wxString value) {
 			stream->SetSourceVideoSize(ffmpeg.GetVideoSize());
 			stream->SetSourceBitrate(ffmpeg.GetBitrate(stIdx));
 			stream->SetSourceAspectRatio(ffmpeg.GetFrameAspectRatio());
-		} else
-			continue;
+		}
 		m_streams.Insert(stream, stIdx);
 	}
 	return ok;
@@ -358,7 +434,7 @@ bool Vob::HasAudio() {
 	return false;
 }
 
-
+/** Returns count of audio streams */
 unsigned int Vob::GetAudioStreamCount() {
 	unsigned int cnt = 0;
 	for (StreamArray::iterator it = m_streams.begin(); it != m_streams.end(); it++)
@@ -367,10 +443,20 @@ unsigned int Vob::GetAudioStreamCount() {
 	return cnt;
 }
 
+/** Returns count of subtitle streams */
 unsigned int Vob::GetSubtitleStreamsCount() {
-	return m_menu ? 1 : GetSubtitles().GetCount();
+	unsigned int cnt = 0;
+	if (!m_menu) {
+		for (StreamArray::iterator it = m_streams.begin(); it != m_streams.end(); it++)
+			if ((*it)->GetType() == stSUBTITLE)
+				cnt++;
+		cnt += GetSubtitles().GetCount();
+	} else
+		cnt = 1;
+	return cnt;
 }
 
+/** Adds the given audio file to the vob */
 bool Vob::AddAudioFile(wxString filename) {
 	wxFfmpegMediaDecoder ffmpeg;
 	if (!ffmpeg.Load(filename))
@@ -388,11 +474,13 @@ bool Vob::AddAudioFile(wxString filename) {
 	return true;
 }
 
+/** Removes audio file with given index from the vob */
 void Vob::RemoveAudioFile(int index) {
 	m_streams.RemoveAt(m_streams.GetCount() - m_audioFilenames.GetCount() + index);
 	m_audioFilenames.RemoveAt(index);
 }
 
+/** Returns the time of given chapter */
 long Vob::GetChapterTime(int chapter) {
 	long time = 0;
 	if (chapter > 0) {
@@ -419,6 +507,43 @@ long Vob::GetChapterTime(int chapter) {
 	return time;
 }
 
+/** Updates pad values to keep aspect ratio*/
+void Vob::UpdatePad(AspectRatio aspectRatio) {
+	int padx = 0;
+	int pady = 0;
+	Stream* videoSt = GetVideoStream();
+	VideoFormat videoFormat = videoSt != NULL ? videoSt->GetVideoFormat() : vfCOPY;
+	if (GetKeepAspectRatio() &&  CalcPad(padx, pady, videoFormat, aspectRatio, m_crop)) {
+		m_pad[0] = m_pad[1] = padx;
+		m_pad[2] = m_pad[3] = pady;
+	}
+}
+
+/** Calculates pad values to keep aspect ratio*/
+bool Vob::CalcPad(int& padx, int& pady, VideoFormat videoFormat, AspectRatio aspectRatio, const vector<int>& crop) {
+	Stream* videoSt = GetVideoStream();
+	if (videoSt != NULL && videoSt->GetSourceAspectRatio() <= 0)
+		return false;
+	padx = 0;
+	pady = 0;
+	if (videoFormat != vfCOPY && aspectRatio != arAUTO) {
+		float oldAspect = videoSt->GetSourceAspectRatio();
+		wxSize oldFrameSize = videoSt->GetSourceVideoSize();
+		int cropX = crop[0] + crop[1];
+		int cropY = crop[2] + crop[3];
+		if (cropX + cropY > 0 && oldFrameSize.x > cropX && oldFrameSize.y > cropY)
+			oldAspect *= ((float) oldFrameSize.y)/oldFrameSize.x*(oldFrameSize.x - cropX)/(oldFrameSize.y - cropY);
+		float aspect = GetFrameAspectRatio(videoFormat, aspectRatio);
+		wxSize frameSize = GetFrameSize(videoFormat);
+		if (aspect > oldAspect)
+			padx = lround(((double)frameSize.x)*(1.0 - oldAspect/aspect)/2);
+		else if (aspect < oldAspect)
+			pady = lround(((double)frameSize.x)*(1.0 - aspect/oldAspect)/2);
+	}
+	return true;
+}
+
+/** Returns URI of video frame */
 wxString Vob::GetVideoFrameUri(int chapter, long position) {
 	wxString uri = GetFilename();
 	if (position == -1 && chapter > 0)
@@ -428,6 +553,14 @@ wxString Vob::GetVideoFrameUri(int chapter, long position) {
 	return uri;
 }
 
+/** Returns video stream parameters */
+Stream* Vob::GetVideoStream() {
+	for (int i = 0; i < (int)GetStreams().GetCount(); i++)
+		if (GetStreams()[i]->GetType() == stVIDEO)
+			return GetStreams()[i];
+	return NULL;
+}
+
 wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 	wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("vob"));
 	wxString fname = GetFilename();
@@ -467,9 +600,13 @@ wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 				continue;
 			wxSvgXmlNode* streamNode = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, streamName);
 			streamNode->AddProperty(wxT("format"), wxString::Format(wxT("%d"), stream->GetDestinationFormat()));
-			if (stream->GetType() == stVIDEO && GetInterlaced()) {
-				streamNode->AddProperty(wxT("interlaced"), wxT("1"));
-				streamNode->AddProperty(wxT("firstField"), wxString::Format(wxT("%d"), GetFirstField()));
+			if (stream->GetType() == stVIDEO) {
+				if (GetInterlaced()) {
+					streamNode->AddProperty(wxT("interlaced"), wxT("1"));
+					streamNode->AddProperty(wxT("firstField"), wxString::Format(wxT("%d"), GetFirstField()));
+				}
+				if (!GetKeepAspectRatio() && stream->GetVideoFormat() > vfCOPY)
+					streamNode->AddProperty(wxT("keepAspect"), wxT("0"));
 			}
 			node->AddChild(streamNode);
 		}
@@ -487,7 +624,7 @@ wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 		}
 		if (GetDoNotTranscode())
 			node->AddProperty(wxT("doNotTranscode"), wxT("1"));
-		if (m_pad.size() == 4 && m_pad[0] + m_pad[1] + m_pad[2] + m_pad[3] > 0) {
+		if (!m_keepAspectRatio && m_pad[0] + m_pad[1] + m_pad[2] + m_pad[3] > 0) {
 			wxSvgXmlNode* padNode = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("pad"));
 			if (m_pad[0] > 0)
 				padNode->AddProperty(wxT("left"), wxString::Format(wxT("%d"), m_pad[0]));
@@ -499,7 +636,7 @@ wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 				padNode->AddProperty(wxT("bottom"), wxString::Format(wxT("%d"), m_pad[3]));
 			node->AddChild(padNode);
 		}
-		if (m_crop.size() == 4 && m_crop[0] + m_crop[1] + m_crop[2] + m_crop[3] > 0) {
+		if (m_crop[0] + m_crop[1] + m_crop[2] + m_crop[3] > 0) {
 			wxSvgXmlNode* cropNode = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("crop"));
 			if (m_crop[0] > 0)
 				cropNode->AddProperty(wxT("left"), wxString::Format(wxT("%d"), m_crop[0]));
@@ -511,6 +648,11 @@ wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 				cropNode->AddProperty(wxT("bottom"), wxString::Format(wxT("%d"), m_crop[3]));
 			node->AddChild(cropNode);
 		}
+		// start & recording time
+		if (GetStartTime() > 0)
+			node->AddProperty(wxT("startTime"), wxString::Format(wxT("%f"), GetStartTime()));
+		if (GetRecordingTime() > 0)
+			node->AddProperty(wxT("recordingTime"), wxString::Format(wxT("%f"), GetRecordingTime()));
 	}
 	
 	if (GetPause() != 0) {
@@ -553,9 +695,10 @@ wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 	return node;
 }
 
-bool Vob::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci) {
+bool Vob::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci, bool menu) {
 	wxString val;
 	long lval;
+	double dval;
 
 	node->GetPropVal(wxT("file"), &val);
 	if (val.length() > 0) {
@@ -596,6 +739,8 @@ bool Vob::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci) {
 				if (child->GetPropVal(wxT("firstField"), &val) && val.length() > 0 && val.ToLong(&lval))
 					m_firstField = (FirstField) lval;
 			}
+			if (child->GetPropVal(wxT("keepAspect"), &val) && val == wxT("0"))
+				m_keepAspectRatio = false;
 		} else if (child->GetName() == wxT("audio")) {
 			if (child->GetChildren() != NULL && child->GetChildren()->GetContent().length() > 0) {
 				val = child->GetChildren()->GetContent();
@@ -624,11 +769,13 @@ bool Vob::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci) {
 			cell->PutXML(child);
 			m_cells.push_back(cell);
 		} else if (child->GetName() == wxT("pad")) {
+			m_pad.clear();
 			m_pad.push_back(child->GetPropVal(wxT("left"), &val) && val.ToLong(&lval) ? lval : 0);
 			m_pad.push_back(child->GetPropVal(wxT("right"), &val) && val.ToLong(&lval) ? lval : 0);
 			m_pad.push_back(child->GetPropVal(wxT("top"), &val) && val.ToLong(&lval) ? lval : 0);
 			m_pad.push_back(child->GetPropVal(wxT("bottom"), &val) && val.ToLong(&lval) ? lval : 0);
 		} else if (child->GetName() == wxT("crop")) {
+			m_crop.clear();
 			m_crop.push_back(child->GetPropVal(wxT("left"), &val) && val.ToLong(&lval) ? lval : 0);
 			m_crop.push_back(child->GetPropVal(wxT("right"), &val) && val.ToLong(&lval) ? lval : 0);
 			m_crop.push_back(child->GetPropVal(wxT("top"), &val) && val.ToLong(&lval) ? lval : 0);
@@ -651,9 +798,13 @@ bool Vob::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci) {
 		else if (val.ToLong(&lval))
 			m_pause = int(lval);
 	}
+	m_startTime = node->GetPropVal(wxT("startTime"), &val) && val.ToDouble(&dval) ? dval : 0;
+	m_recordingTime = node->GetPropVal(wxT("recordingTime"), &val) && val.ToDouble(&dval) ? dval : -1;
+		
 	if (node->GetPropVal(wxT("doNotTranscode"), &val) && val == wxT("1"))
 		SetDoNotTranscode(true);
 	node->GetPropVal(wxT("chapters"), &m_chapters);
+	
 	return true;
 }
 
@@ -833,7 +984,7 @@ wxSvgXmlNode* Pgc::GetXML(DVDFileType type, DVD* dvd, wxString nextTitle) {
 	return node;
 }
 
-bool Pgc::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci) {
+bool Pgc::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci, bool menu) {
 	WX_CLEAR_ARRAY(GetVobs());
 
 	wxString val;
@@ -847,22 +998,22 @@ bool Pgc::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci) {
 	node->GetPropVal(wxT("palette"), &m_palette);
 
 	wxSvgXmlNode* child = node->GetChildren();
-	Menu* menu = NULL; // old
+	Menu* menuObj = NULL; // old
 
 	while (child) {
 		if (child->GetName() == wxT("spumux")) {
 			// get spunode (old)
-			menu = new Menu(dvd, tsi, pgci);
-			menu->PutXML(child);
+			menuObj = new Menu(dvd, tsi, pgci);
+			menuObj->PutXML(child);
 		} else if (child->GetName() == wxT("vob")) {
 			// get vob
 			Vob* vob;
-			if (menu)
-				vob = new Vob(menu);
+			if (menuObj)
+				vob = new Vob(menuObj);
 			else
 				vob = new Vob(wxT(""));
 			menu = NULL;
-			if (vob->PutXML(child, dvd, tsi, pgci))
+			if (vob->PutXML(child, dvd, tsi, pgci, menu))
 				GetVobs().Add(vob);
 			else {
 				delete vob;
@@ -1128,7 +1279,7 @@ bool Menus::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi) {
 			m_video.PutXML(child, true);
 		else if (child->GetName() == wxT("pgc")) {
 			Pgc* pgc = new Pgc;
-			if (pgc->PutXML(child, dvd, tsi, GetCount()))
+			if (pgc->PutXML(child, dvd, tsi, GetCount(), true))
 				Add(pgc);
 			else {
 				delete pgc;
@@ -1191,7 +1342,7 @@ bool Titles::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi) {
 			GetVideo().PutXML(child, false);
 		} else if (child->GetName() == wxT("pgc")) {
 			Pgc* pgc = new Pgc;
-			if (pgc->PutXML(child, dvd, tsi, GetCount()))
+			if (pgc->PutXML(child, dvd, tsi, GetCount(), false))
 				Add(pgc);
 			else {
 				delete pgc;
@@ -1201,6 +1352,19 @@ bool Titles::PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi) {
 		}
 		child = child->GetNext();
 	}
+	// set aspect ratio if it isn't set
+	if (GetVideo().GetAspect() == arAUTO && GetCount() > 0 && Item(0)->GetVobs().GetCount() > 0) {
+		if (Item(0)->GetVobs()[0]->GetSlideshow() != NULL) {
+			GetVideo().SetAspect(Item(0)->GetVobs()[0]->GetSlideshow()->GetAspectRatio());
+		} else {
+			Stream* videoSt = Item(0)->GetVobs()[0]->GetVideoStream();
+			float aspect = videoSt->GetSourceAspectRatio();
+			if (aspect > 0)
+				GetVideo().SetAspect(aspect < 1.4 ? ar4_3 : ar16_9);
+			else
+				GetVideo().SetAspect(dvd->GetAspectRatio());
+		}
+	}
 	return true;
 }
 
@@ -1902,6 +2066,15 @@ wxArrayString DVD::GetAudioFormatLabels(bool copy, bool none) {
 	return formats;
 }
 
+wxArrayString DVD::GetSubtitleFormatLabels(bool copy, bool none) {
+	wxArrayString formats;
+	if (none)
+		formats.Add(_("None"));
+	if (copy)
+		formats.Add(_("Copy"));
+	return formats;
+}
+
 wxArrayString DVD::GetAspectRatioLabels(bool autom) {
 	wxArrayString formats;
 	if (autom)
@@ -2210,7 +2383,7 @@ const wxString charsetsArray[] = {
 	wxT("UTF-7"), wxT("UTF-8"), wxT("UTF-16"), wxT("UTF-16BE"), wxT("UTF-16LE"),
 	wxT("UTF-32"), wxT("UTF-32BE"), wxT("UTF-32LE"), wxT("VISCII")
 };
-static wxArrayString s_charsets(80, charsetsArray);
+static wxArrayString s_charsets(94, charsetsArray);
 
 wxArrayString& DVD::GetCharsets() {
 	return s_charsets;
