@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Structure (Titles/Menus)
 // Author:      Alex Thuering
 // Created:	29.01.2003
-// RCS-ID:      $Id: DVD.h,v 1.64 2010/10/10 15:54:55 ntalex Exp $
+// RCS-ID:      $Id: DVD.h,v 1.67 2011/03/10 20:01:27 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -108,6 +108,21 @@ public:
 	double GetFontSize() { return m_fontSize; }
 	void SetFontSize(double value) { m_fontSize = value; }
 	
+    wxColour GetFillColour() { return m_fillColour; }
+    void SetFillColour(const wxColour& value) { m_fillColour = value; }
+    
+    wxColour GetOutlineColour() { return m_outlineColour; }
+    void SetOutlineColour(const wxColour& value) { m_outlineColour = value; }
+    
+    double GetOutlineThickness() { return m_outlineThickness; }
+    void SetOutlineThickness(double value) { m_outlineThickness = value; }
+    
+    wxColour GetShadowColour() { return m_shadowColour; }
+    void SetShadowColour(const wxColour& value) { m_shadowColour = value; }
+    
+    wxPoint GetShadowOffset() { return m_shadowOffset; }
+    void SetShadowOffset(const wxPoint& value) { m_shadowOffset = value; }
+	
 	wxAlignment GetAlignment() { return m_alignment; }
 	void SetAlignment(wxAlignment value) { m_alignment = value; }
 	
@@ -129,17 +144,17 @@ public:
 	double GetMovieFps() { return m_movieFps; }
 	void SetMovieFps(double value) { m_movieFps = value; }
 	
-	int GetMovieWidth() { return m_movieWidth; }
-	void SetMovieWidth(int value) { m_movieWidth = value; }
+	wxSize GetMovieSize() { return m_movieSize; }
+	void SetMovieSize(wxSize value) { m_movieSize = value; }
 	
-	int GetMovieHeight() { return m_movieHeight; }
-	void SetMovieHeight(int value) { m_movieHeight = value; }
+	AspectRatio GetAspectRatio() { return m_aspectRatio; }
+	void SetAspectRatio(AspectRatio value) { m_aspectRatio = value; }
 	
 	wxSvgXmlNode* GetXML(DVDFileType type);
 	bool PutXML(wxSvgXmlNode* node);
 	
 	/** saves a configuration for spumux */
-	bool SaveSpumux(wxString filename);
+	bool SaveSpumux(wxString filename, VideoFormat videoFormat);
 	
 private:
     wxString m_filename;
@@ -147,6 +162,11 @@ private:
     wxString m_fontFamily;
     wxString m_fontStyle;
     double m_fontSize;
+    wxColour m_fillColour;
+    wxColour m_outlineColour;
+    double m_outlineThickness;
+    wxColour m_shadowColour;
+    wxPoint m_shadowOffset;
     wxAlignment m_alignment;
     int m_leftMargin;
     int m_rightMargin;
@@ -154,8 +174,8 @@ private:
     int m_bottomMargin;
     double m_subtitleFps;
     double m_movieFps;
-    int m_movieWidth;
-    int m_movieHeight;
+    wxSize m_movieSize;
+    AspectRatio m_aspectRatio;
 };
 
 WX_DEFINE_ARRAY(TextSub*, TextSubArray);
@@ -235,46 +255,70 @@ public:
 	/** Returns count of audio streams */
 	unsigned int GetAudioStreamCount();
 	
-    /** returns array with audio tracks */
+    /** Returns array with audio tracks */
 	const wxArrayString& GetAudioFilenames() { return m_audioFilenames; }
-	/** adds the given audio file to the vob */
+	/** Adds the given audio file to the vob */
 	bool AddAudioFile(wxString filename);
-	/** removes audio file with given index from the vob */
+	/** Removes audio file with given index from the vob */
 	void RemoveAudioFile(int index);
 	
-    /** returns array with subtitle tracks */
+    /** Returns array with subtitle tracks */
     TextSubArray& GetSubtitles() { return m_subtitles; }
 	/** Returns count of subtitle streams */
 	unsigned int GetSubtitleStreamsCount();
 	
-    /** sets file name of tempory vob file */
+    /** Sets file name of tempory vob file */
 	void SetTmpFilename(wxString value) { m_tmpFilename = value; }
-    /** returns file name of tempory vob file */
+    /** Returns file name of tempory vob file */
 	wxString GetTmpFilename() { return m_tmpFilename; }
 	
-    /** sets chapter list */
+    /** Sets chapter list */
 	void SetChapters(wxString value) { m_chapters = value; }
-    /** returns chapter list */
+    /** Returns chapter list */
 	wxString GetChapters() { return m_chapters; }
-	/** returns the time of given chapter */
+	/** Returns the time of given chapter */
 	long GetChapterTime(int chapter);
 	
-    /** sets pause */
+    /** Sets pause */
 	void SetPause(int value) { m_pause = value; }
-    /** returns pause */
+    /** Returns pause */
 	int GetPause() { return m_pause; }
 	
+	/** Sets start time */
+	void SetStartTime(double value) { m_startTime = value; }
+	/** Returns start time */
+	double GetStartTime() { return m_startTime; }
+	
+	/** Sets recording time */
+	void SetRecordingTime(double value) { m_recordingTime = value; }
+	/** Returns recording time */
+	double GetRecordingTime() { return m_recordingTime; }
+	
     /** Returns cells */
 	vector<Cell*>& GetCells() { return m_cells; }
+	
+	/** Returns true if aspect ratio must be retained */
+	bool GetKeepAspectRatio() { return m_keepAspectRatio; }
+	/** Sets if aspect ratio must be retained */
+	void SetKeepAspectRatio(bool keepAspectRatio) { m_keepAspectRatio = keepAspectRatio; }
+	
 	/** Returns pad values */
 	vector<int>& GetPad() { return m_pad; }
+	/** Updates pad value to keep aspect ratio*/
+	void UpdatePad(AspectRatio aspectRatio);
+	/** Calculates pad value to keep aspect ratio*/
+	bool CalcPad(int& padx, int& pady, VideoFormat videoFormat, AspectRatio aspectRatio, const vector<int>& crop);
 	/** Returns crop values */
 	vector<int>& GetCrop() { return m_crop; }
 	
+	/** Returns encode interlaced flag */
 	bool GetInterlaced() { return m_interlaced; }
+	/** Sets encode interlaced flag */
 	void SetInterlaced(bool interlaced) { m_interlaced = interlaced; }
 	
+	/** Returns first field (interlacing) flag */
 	FirstField GetFirstField() { return m_firstField;; }
+	/** Sets first field (interlacing) flag */
 	void SetFirstField(FirstField firstField) { m_firstField = firstField; }
 
 	void SetMenu(Menu* menu) { m_menu = menu; }
@@ -283,20 +327,22 @@ public:
     void SetSlideshow(Slideshow* slideshow) { m_slideshow = slideshow; }
 	Slideshow* GetSlideshow() { return m_slideshow; }
 	
-	/** returns duration of video in ms */
+	/** Returns duration of video in ms */
 	inline double GetDuration() { return m_duration; }
-	/** returns URI of video frame */
+	/** Returns URI of video frame */
 	wxString GetVideoFrameUri(int chapter, long position = -1);
-    /** returns array with stream parameters */
+    /** Returns array with stream parameters */
     StreamArray& GetStreams() { return m_streams; }
+    /** Returns video stream parameters */
+    Stream* GetVideoStream();
 	
-    /** returns true if the source video file must not be remultiplexed/transcoded */
+    /** Returns true if the source video file must not be remultiplexed/transcoded */
    	inline bool GetDoNotTranscode() { return m_doNotTranscode; }
-  	/** sets if the source video file must not be remultiplexed/transcoded */
+  	/** Sets if the source video file must not be remultiplexed/transcoded */
    	inline void SetDoNotTranscode(bool doNotTranscode) { m_doNotTranscode = doNotTranscode; }
 	
 	wxSvgXmlNode* GetXML(DVDFileType type, DVD* dvd);
-	bool PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci);
+	bool PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci, bool menu);
 	
 	/**
 	 * Returns size in KB
@@ -318,21 +364,26 @@ private:
 	wxArrayString m_audioFilenames;
     TextSubArray m_subtitles;
 	wxString m_tmpFilename;
+	
 	wxString m_chapters;
 	int m_pause;
 	vector<Cell*> m_cells;
-	vector<int> m_pad;
-	vector<int> m_crop;
-	bool m_interlaced;
-	FirstField m_firstField;
+	
 	Menu* m_menu;
     Slideshow* m_slideshow;
-	static unsigned int GetFileSize(wxString filename);
 	
 	double m_duration;
+	double m_startTime;
+	double m_recordingTime;
 	StreamArray m_streams;
 	bool m_doNotTranscode;
+	bool m_interlaced;
+	FirstField m_firstField;
+	bool m_keepAspectRatio;
+	vector<int> m_pad;
+	vector<int> m_crop;
 	void Init(Menu* menu = NULL, Slideshow* slideshow = NULL);
+	static unsigned int GetFileSize(wxString filename);
 };
 
 WX_DEFINE_ARRAY(Vob*, VobArray);
@@ -362,7 +413,7 @@ public:
     unsigned int GetChapterCount();
 	
 	wxSvgXmlNode* GetXML(DVDFileType type, DVD* dvd, wxString nextTitle);
-	bool PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci);
+	bool PutXML(wxSvgXmlNode* node, DVD* dvd, int tsi, int pgci, bool menu);
 	
 private:
 	VobArray m_vobs;
@@ -709,6 +760,7 @@ public:
 	
 	static wxArrayString GetVideoFormatLabels(bool copy = false, bool none = false, bool menu = false);
 	static wxArrayString GetAudioFormatLabels(bool copy = false, bool none = false);
+	static wxArrayString GetSubtitleFormatLabels(bool copy = false, bool none = false);
 	static wxArrayString GetAspectRatioLabels(bool autom = false);
 	static wxArrayString GetCapacityLabels();
 	static wxArrayString GetVideoBitrateLabels();
