@@ -3,7 +3,7 @@
 // Purpose:     Stores a DVD Action
 // Author:      Alex Thuering
 // Created:     03.09.2009
-// RCS-ID:      $Id: DVDAction.cpp,v 1.20 2010/09/05 10:54:03 ntalex Exp $
+// RCS-ID:      $Id: DVDAction.cpp,v 1.21 2011/02/20 18:52:16 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -80,7 +80,7 @@ wxString DVDAction::AsString(DVD* dvd, bool translate) {
 				action += _("jump to menu") + wxString(wxT(" entry ")) + entry;
 			}
 		} else
-			action += wxT(" entry ") + entry;
+			action += wxT(" menu entry ") + entry;
 	} else {
 		int n = pgci + 1;
 		if (m_vmg && !menu && !translate) // use sequence number of title for jumping to title from vmg
