@@ -3,7 +3,7 @@
 // Purpose:     Languages dialog
 // Author:      Alex Thuering
 // Created:	23.05.2003
-// RCS-ID:      $Id: Languages.h,v 1.5 2010/04/03 21:45:27 ntalex Exp $
+// RCS-ID:      $Id: Languages.h,v 1.6 2010/06/30 20:05:05 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -15,7 +15,7 @@
 #include <wxVillaLib/PropDlg.h>
 
 int ChooseLanguage();
-wxArrayString GetLangNames();
+const wxArrayString& GetLangNames();
 wxString GetLangName(int code);
 int GetLangCode(int index);
 const BitmapArray& GetLangBitmaps();
