@@ -3,7 +3,7 @@
 // Purpose:     Main window
 // Author:      Alex Thuering
 // Created:     10.10.2003
-// RCS-ID:      $Id: MainWin.cpp,v 1.143 2010/09/05 10:57:29 ntalex Exp $
+// RCS-ID:      $Id: MainWin.cpp,v 1.144 2011/02/22 18:42:05 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -699,8 +699,7 @@ bool MainWin::Open(wxString filename) {
 
 void MainWin::ShowOpenDlg() {
 	wxFileDialog dlg(this, _("Open a DVDStyler project file"), m_lastDir, wxT(""),
-			_("Project files") + wxString(wxT(" (*.dvds;*.xml)|*.dvds;*.xml|"))
-			+ _("All Files") + wxString(wxT(" (*.*)|*.*")), wxFD_OPEN);
+			_("Project files") + wxString(wxT(" (*.dvds;*.xml)|*.dvds;*.xml|")), wxFD_OPEN);
 	dlg.Centre();
 	if (dlg.ShowModal() == wxID_OK)
 		Open(dlg.GetPath());
@@ -984,11 +983,11 @@ void MainWin::AddFileDialog(bool asChapter) {
 			+ _("Video Files") + wxString::Format(wxT(" (%s)|%s|"), video.c_str(), video.c_str())
 			+ _("Subtitle Files") + wxString::Format(wxT(" (%s)|%s|"), subtitle.c_str(), subtitle.c_str())
 			+ _("Image Files") + wxString::Format(wxT(" %s"), wxImage::GetImageExtWildcard().c_str())
-			+ _("All files") + wxString(wxT(" (*.*)|*.*|"));
+			+ wxString(wxT("|")) + _("All Files") + wxString(wxT(" (*.*)|*.*|"));
 	if (asChapter)
 		wildcard = _("Video Files") + wxString::Format(wxT(" (%s)|%s|"), video.c_str(), video.c_str())
 				+ _("Image Files") + wxString::Format(wxT(" %s"), wxImage::GetImageExtWildcard().c_str())
-				+ _("All files") + wxString(wxT(" (*.*)|*.*|"));
+				+ wxString(wxT("|")) + _("All Files") + wxString(wxT(" (*.*)|*.*|"));
 	wxFileDialog dialog(this, _("Choose a file"), wxT(""), wxT(""), wildcard, wxFD_OPEN | wxFD_MULTIPLE);
 	// set file browser dir
 	dialog.SetDirectory(s_config.GetLastAddDir() + wxFILE_SEP_PATH);
