@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Menu
 // Author:      Alex Thuering
 // Created:	    04.11.2003
-// RCS-ID:      $Id: Menu.h,v 1.48 2010/05/03 06:03:20 ntalex Exp $
+// RCS-ID:      $Id: Menu.h,v 1.54 2010/09/05 10:53:06 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -35,16 +35,23 @@ public:
 	 * @param videoFormat The video format (PAL/NTSC)
 	 * @param aspectRatio The aspectRatio (4:3/16:9)
 	 */
-	Menu(DVD* dvd, int tsi, VideoFormat videoFormat = vfPAL, AspectRatio aspectRatio = ar4_3);
+	Menu(DVD* dvd, int tsi, int pgci, VideoFormat videoFormat = vfPAL, AspectRatio aspectRatio = ar4_3);
 	~Menu();
 	
 	inline wxSVGDocument* GetSVG() { return m_svg; }
 	
+	/** Returns video format */
 	inline VideoFormat GetVideoFormat() { return m_videoFormat; }
+	/** Sets video format */
 	void SetVideoFormat(VideoFormat format);
+	
 	wxSize GetResolution();
+	wxSize GetFrameResolution();
+	
+	/** Returns aspect ratio */
 	inline AspectRatio GetAspectRatio() { return m_aspectRatio; }
-	inline void SetAspectRatio(AspectRatio aspectRatio) { m_aspectRatio = aspectRatio; }
+	/** Sets aspect ratio */
+	void SetAspectRatio(AspectRatio aspectRatio);
 	
 	inline wxString GetStartTime() { return m_startTime; }
 	inline void 	SetStartTime(wxString time) { m_startTime = time; }
@@ -69,13 +76,21 @@ public:
 	MenuObject* GetObject(int index) { return m_objects[index]; }
 	/** Returns menu object with given id */
 	MenuObject* GetObject(wxString id);
+	/** Returns index of menu object with given id */
+	int GetIndexOfObject(wxString id);
 	/** Removes menu object with given id */
 	void RemoveObject(wxString id);
+	/** Moves object to front */
 	inline void ObjectToFront(wxString id) { MenuObject* obj = GetObject(id); if (obj) obj->ToFront(); }
+	/** Moves object forward */
 	inline void ObjectForward(wxString id) { MenuObject* obj = GetObject(id); if (obj) obj->Forward(); }
+	/** Moves object backward */
 	inline void ObjectBackward(wxString id) { MenuObject* obj = GetObject(id); if (obj) obj->Backward(); }
+	/** Moves object to back */
 	inline void ObjectToBack(wxString id) { MenuObject* obj = GetObject(id); if (obj) obj->ToBack(); }
+	/** Returns true if the object with given id is first */
 	inline bool IsFirstObject(wxString id) { MenuObject* obj = GetObject(id); return obj && obj->IsFirst(); }
+	/** Returns true if the object with given id is last */
 	inline bool IsLastObject(wxString id) { MenuObject* obj = GetObject(id); return obj && obj->IsLast(); }
 	/** Returns true if button with given id is the first button */
 	bool IsFirstButton(wxString id);
@@ -85,14 +100,21 @@ public:
 	inline unsigned int GetActionsCount() { return m_actions.Count(); }
 	inline DVDAction* GetAction(int index) { return m_actions[index]; }
 	
+	/** Returns true if the last selected button must be remembered */
+	inline bool GetRememberLastButton() { return m_rememberLastButton; }
+	/** Sets if the last selected button must be remembered */
+	inline void SetRememberLastButton(bool rememberLastButton) { m_rememberLastButton = rememberLastButton; }
+
 	void SetTransparentColor();
 	wxImage GetImage(int width, int height);
+	wxImage GetBackgroundImage();
 	
 	bool ReduceColours();
-	wxImage* GetImages();
+	/** Returns subPictures */
+	wxImage* GetSubPictures(SubStreamMode mode);
 	wxSVGDocument* GetBackgroundSVG();
 	
-	bool SaveSpumux(wxString fileName, wxString btFile, wxString hlFile, wxString selFile);
+	bool SaveSpumux(wxString fileName, SubStreamMode mode, wxString btFile, wxString hlFile, wxString selFile);
 	wxSvgXmlNode* GetXML(DVDFileType type, wxSvgXmlNode* node = NULL);
 	bool PutXML(wxSvgXmlNode* node);
 	/** Stores object data to string */
@@ -101,9 +123,11 @@ public:
 	void Deserialize(const wxString& data);
 	
 	wxString AddObject(wxSvgXmlNode* node, bool fixPosition = false);
-	
+
+	/** Returns button with given jump action */
+	MenuObject* GetButtonByJumpAction(int tsi, int pgci);
 	/** Updates image in buttons with given jump action */
-	void UpdateButtonImageFor(int actionTsi, int actionPgci, DVD* dvd);
+	bool UpdateButtonImageFor(int actionTsi, int actionPgci, DVD* dvd);
 	/** Fix coordinates of buttons if they are out of range */
 	void FixButtonCoordinates();
 	
@@ -112,6 +136,7 @@ public:
 private:
 	DVD* m_dvd;
 	int m_tsi;
+	int m_pgci;
 	wxSVGDocument* m_svg;
 	VideoFormat m_videoFormat;
 	AspectRatio m_aspectRatio;
@@ -119,6 +144,7 @@ private:
 	wxColour 	m_transpColour;
 	MenuObjects m_objects;
 	DVDActions m_actions;
+	bool m_rememberLastButton;
 	void SetBackground(wxSVGSVGElement* svg, wxString fileName);
 	wxString GetBackground(wxSVGSVGElement* svg);
 	void FixPosition(MenuObject* obj); /** for copy & paste */
@@ -126,7 +152,8 @@ private:
 	void RemoveChangeable(wxSVGElement* element, MenuObject* obj);
 	bool RemoveNotChangeable(wxSVGElement* element, MenuObject* obj);
 	wxSVGSVGElement* GetSVGCopy(MenuDrawType drawType = mdALL);
-	wxImage RenderImage(MenuDrawType drawType, int width, int height);
+	wxImage RenderImage(MenuDrawType drawType, int width, int height, const wxSVGRect* rect = NULL,
+			bool preserveAspectRatio = false);
 	wxImage GetImage(MenuDrawType drawType, int width = -1, int height = -1);
 };
 
