@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Menu Object
 // Author:      Alex Thuering
 // Created:	04.11.2006
-// RCS-ID:      $Id: MenuObject.cpp,v 1.19 2009/12/30 16:56:48 ntalex Exp $
+// RCS-ID:      $Id: MenuObject.cpp,v 1.24 2010/09/05 10:54:46 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -19,6 +19,8 @@
 #define BUTTONS_DIR wxFindDataDirectory(_T("buttons"))
 #define OBJECTS_DIR wxFindDataDirectory(_T("objects"))
 
+const wxString TRANS_ELEM_ID = wxT("s_trans");
+
 ///////////////////////////// MenuObject /////////////////////////////////////
 
 MenuObject::MenuObject(Menu* menu, bool vmg, wxString fileName, int x, int y, wxString param):
@@ -34,6 +36,7 @@ MenuObject::MenuObject(Menu* menu, bool vmg, wxString fileName, int x, int y, wx
 	} else {
 		m_deleteSVG = false;
 		m_svg = menu->GetSVG();
+		m_menu = menu;
 	}
 	m_use = NULL;
 	m_symbol = NULL;
@@ -42,6 +45,8 @@ MenuObject::MenuObject(Menu* menu, bool vmg, wxString fileName, int x, int y, wx
 	m_previewHighlighted = false;
 
 	m_button = false;
+	m_displayVideoFrame = true;
+	m_customVideoFrame = false;
 
 	m_defaultSize = true;
 	m_defaultWidth.value = m_defaultHeight.value = 0;
@@ -80,32 +85,6 @@ bool MenuObject::Init(wxString fileName, int x, int y, wxString param) {
 	if (m_id.length()) { // load button
 		m_symbol = (wxSVGSVGElement*) m_svg->GetElementById(wxT("s_") + m_id);
 		m_use = (wxSVGUseElement*) m_svg->GetElementById(m_id);
-
-		// fix old format (1.5b1 - 1.5b3)
-		if (m_symbol && m_symbol->GetDtd() == wxSVG_SYMBOL_ELEMENT) {
-			// replace symbol element with svg element
-			wxSVGSymbolElement* symbol = (wxSVGSymbolElement*) m_symbol;
-			m_symbol = new wxSVGSVGElement();
-			m_symbol->SetId(symbol->GetId());
-			m_symbol->SetViewBox(symbol->GetViewBox());
-			m_symbol->SetPreserveAspectRatio(symbol->GetPreserveAspectRatio());
-			wxSvgXmlElement* child = symbol->GetChildren();
-			while (child) {
-				m_symbol->AddChild(child->CloneNode());
-				child = child->GetNext();
-			}
-			wxSvgXmlElement* parent = symbol->GetParent();
-			parent->RemoveChild(symbol);
-			parent->AddChild(m_symbol);
-			// remove id prefix
-			child = m_symbol->GetChildren();
-			while (child) {
-				wxSVGElement& elem = *(wxSVGElement*) child;
-				if (elem.GetId().length())
-					elem.SetId(elem.GetId().AfterFirst(wxT('_')).AfterFirst(wxT('_')));
-				child = child->GetNext();
-			}
-		}
 	} else
 		// create new button
 		m_id = GenerateId(m_button ? wxT("button") : wxT("obj"));
@@ -276,13 +255,42 @@ void MenuObject::AddUse(wxString id, int x, int y, int width, int height) {
 		m_svg->GetElementById(wxT("objects"))->AppendChild(m_use);
 }
 
+void MenuObject::SetScale(double scaleX, double scaleY) {
+	wxSVGGElement* transElem = (wxSVGGElement*) m_symbol->GetElementById(TRANS_ELEM_ID);
+	if (transElem == NULL) {
+		if (scaleX == 1 && scaleY == 1)
+			return;
+		// add transElem
+		transElem = new wxSVGGElement();
+		transElem->SetId(TRANS_ELEM_ID);
+		m_symbol->AddChild(transElem);
+		// move all children elements in gElem
+		while (m_symbol->GetFirstChild() != transElem) {
+			wxSvgXmlNode* elem = m_symbol->GetFirstChild();
+			m_symbol->RemoveChild(elem);
+			transElem->AppendChild(elem);
+		}
+	}
+	wxSVGTransformList transList;
+	if (scaleX != 1 || scaleY != 1) {
+		wxSVGTransform transform;
+		transform.SetScale(scaleX, scaleY);
+		transList.Add(transform);
+	}
+	// copy old values
+	const wxSVGTransformList& oldTransList = transElem->GetTransform().GetBaseVal();
+	for (unsigned int i = 0; i < oldTransList.size(); i++)
+		if (i > 0 || oldTransList[i].GetType() != wxSVG_TRANSFORM_SCALE)
+			transList.Add(oldTransList[i]);
+	transElem->SetTransform(transList);
+}
+
 wxString MenuObject::GetId(bool translate) {
 	if (!translate)
 		return m_id;
 	long l = 0;
 	m_id.Mid(IsButton() ? 6 : 3).ToLong(&l);
-	return (IsButton() ? _("button") : _("object")) + wxString::Format(
-			wxT(" %d"), l);
+	return (IsButton() ? _("button") : _("object")) + wxString::Format(wxT(" %d"), l);
 }
 
 int MenuObject::GetX() {
@@ -313,6 +321,26 @@ void MenuObject::SetHeight(int value) {
 	m_use->SetHeight(value);
 }
 
+wxRect MenuObject::GetBBox() {
+	return wxRect(GetX(), GetY(), GetWidth(), GetHeight());
+}
+
+wxRect MenuObject::GetFrameBBox(SubStreamMode mode, bool ignorePadding) {
+	double fx = mode == ssmPANSCAN ? 4.0/3 : 1.0;
+	double fy = mode == ssmLETTERBOX ? 0.75 : 1.0;
+	if (m_menu != NULL)
+		fy *= (double) m_menu->GetFrameResolution().GetHeight() / m_menu->GetResolution().GetHeight();
+	int width = round(GetWidth()*fx);
+	int height = round(GetHeight()*fy);
+	if (width % 2 == 1)
+		width++;
+	if (height % 2 == 1)
+		height++;
+	int padY = mode == ssmLETTERBOX && !ignorePadding ? m_menu->GetFrameResolution().GetHeight()*0.125: 0;
+	int cropX = mode == ssmPANSCAN ? m_menu->GetFrameResolution().GetWidth()*0.125 : 0;
+	return wxRect(round((GetX() - cropX)*fx), round(GetY()*fy) + padY, width, height);
+}
+
 unsigned int MenuObject::CalcSize(MenuObjectSize& size, bool width) {
 	unsigned int result = 0;
 	for (unsigned int idx = 0; idx < size.elements.GetCount(); idx++) {
@@ -341,10 +369,6 @@ void MenuObject::FixSize(int& width, int& height) {
 		width = wxMax(width, (int) CalcSize(m_minWidth, true));
 		height = wxMax(height, (int) CalcSize(m_minHeight, false));
 	}
-	if (width % 2 == 1)
-		width++;
-	if (height % 2 == 1)
-		height++;
 }
 
 void MenuObject::UpdateSize() {
@@ -390,8 +414,7 @@ void MenuObject::SetParam(wxString name, wxString value, wxString attrSuffix) {
 	MenuObjectParam* param = GetObjectParam(name);
 	if (!param)
 		return;
-	wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(
-			param->element);
+	wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(param->element);
 	if (!elem)
 		return;
 	if (param->attribute.length()) {
@@ -446,8 +469,7 @@ wxFont MenuObject::GetParamFont(wxString name) {
 	MenuObjectParam* param = GetObjectParam(name);
 	if (!param)
 		return wxFont(20, wxFONTFAMILY_DEFAULT, wxNORMAL, wxNORMAL, false);
-	wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(
-			param->element);
+	wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(param->element);
 	if (!elem)
 		return wxFont(20, wxFONTFAMILY_DEFAULT, wxNORMAL, wxNORMAL, false);
 
@@ -481,13 +503,11 @@ void MenuObject::SetParamFont(wxString name, wxFont value) {
 	MenuObjectParam* param = GetObjectParam(name);
 	if (!param)
 		return;
-	wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(
-			param->element);
+	wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(param->element);
 	if (!elem)
 		return;
 
-	elem->SetAttribute(wxT("font-size"), wxString::Format(wxT("%d"),
-			value.GetPointSize()));
+	elem->SetAttribute(wxT("font-size"), wxString::Format(wxT("%d"), value.GetPointSize()));
 
 	wxString styleStr = wxT("normal");
 	if (value.GetStyle() == wxFONTSTYLE_ITALIC)
@@ -506,6 +526,9 @@ void MenuObject::SetParamFont(wxString name, wxFont value) {
 	elem->SetAttribute(wxT("font-weight"), weightStr);
 
 	elem->SetAttribute(wxT("font-family"), value.GetFaceName());
+	
+	if (elem->GetDtd() == wxSVG_TEXT_ELEMENT)
+		((wxSVGTextElement*) elem)->SetCanvasItem(NULL);
 }
 
 wxColour MenuObject::GetParamColour(wxString name, MenuButtonState state) {
@@ -638,7 +661,7 @@ wxImage MenuObject::GetImage(int maxWidth, int maxHeight) {
 	return m_svg->Render(maxWidth, maxHeight);
 }
 
-wxSvgXmlNode* MenuObject::GetXML(DVDFileType type, DVD* dvd, bool withSVG, int playAllRegister) {
+wxSvgXmlNode* MenuObject::GetXML(DVDFileType type, DVD* dvd, SubStreamMode mode, bool withSVG) {
 	wxString rootName = wxT("object");
 	if (IsButton())
 		rootName = wxT("button");
@@ -672,20 +695,25 @@ wxSvgXmlNode* MenuObject::GetXML(DVDFileType type, DVD* dvd, bool withSVG, int p
 				rootNode->AddChild(paramNode);
 			}
 		}
-		rootNode->AddProperty(_T("id"), GetId());
+		rootNode->AddProperty(wxT("id"), GetId());
 		if (IsDefaultSize())
-			rootNode->AddProperty(_T("defSize"), _T("true"));
+			rootNode->AddProperty(wxT("defSize"), wxT("true"));
+		if (!m_displayVideoFrame)
+			rootNode->AddProperty(wxT("displayVideoFrame"), wxT("false"));
+		if (m_customVideoFrame)
+			rootNode->AddProperty(wxT("customVideoFrame"), wxT("true"));
 		break;
 	case SPUMUX_XML: {
-		int mwidth = (int) m_svg->GetRootElement()->GetWidth().GetBaseVal() - 1;
-		int mheight = (int) m_svg->GetRootElement()->GetHeight().GetBaseVal() - 1;
+		int mwidth = (int) m_menu->GetFrameResolution().GetWidth() - 1;
+		int mheight = (int) m_menu->GetFrameResolution().GetHeight() - 1;
 		rootNode->AddProperty(_T("name"), GetId());
-		rootNode->AddProperty(_T("x0"), wxString::Format(_T("%d"), GetX() > 0 ? GetX() : 0));
-		rootNode->AddProperty(_T("y0"), wxString::Format(_T("%d"), GetY() > 0 ? GetY() : 0));
+		wxRect rect = GetFrameBBox(mode);
+		rootNode->AddProperty(_T("x0"), wxString::Format(_T("%d"), rect.GetX() > 0 ? rect.GetX() : 0));
+		rootNode->AddProperty(_T("y0"), wxString::Format(_T("%d"), rect.GetY() > 0 ? rect.GetY() : 0));
 		rootNode->AddProperty(_T("x1"), wxString::Format(_T("%d"),
-				GetX() + GetWidth() < mwidth ? GetX() + GetWidth() : mwidth));
+				rect.GetX() + rect.GetWidth() < mwidth ? rect.GetX() + rect.GetWidth() : mwidth));
 		rootNode->AddProperty(_T("y1"), wxString::Format(_T("%d"),
-				GetY() + GetHeight() < mheight ? GetY() + GetHeight() : mheight));
+				rect.GetY() + rect.GetHeight() < mheight ? rect.GetY() + rect.GetHeight() : mheight));
 		if (GetDirection(0).length())
 			rootNode->AddProperty(_T("left"), GetDirection(0));
 		if (GetDirection(1).length())
@@ -700,8 +728,11 @@ wxSvgXmlNode* MenuObject::GetXML(DVDFileType type, DVD* dvd, bool withSVG, int p
 		rootNode->AddProperty(_T("name"), GetId());
 		wxString action = GetAction().AsString(dvd);
 		action.Replace(wxT("vmMenu"), wxT("vmgm menu"));
-		if (playAllRegister != -1)
-			action = wxString::Format(wxT("g%d=%d;"), playAllRegister, GetAction().IsPlayAll()) + action;
+		if (dvd->GetPlayAllRegister() != -1)
+			action = wxString::Format(wxT("g%d=%d;"), dvd->GetPlayAllRegister(), GetAction().IsPlayAll()) + action;
+		if (dvd->GetRememberLastButtonRegister() != -1 && !GetAction().GetCustom())
+			action = wxString::Format(GetAction().IsMenu() ? wxT("g%d=0;") : wxT("g%d=button;"),
+					dvd->GetRememberLastButtonRegister()) + action;
 		rootNode->AddChild(new wxSvgXmlNode(wxSVGXML_TEXT_NODE, wxEmptyString, action));
 		break;
 	}
@@ -786,9 +817,11 @@ bool MenuObject::PutXML(wxSvgXmlNode* node) {
 		}
 	}
 
-	node->GetPropVal(_T("id"), &m_id);
-	if (!node->GetPropVal(_T("defSize"), &val) || (val != wxT("true") && val != wxT("1")))
+	node->GetPropVal(wxT("id"), &m_id);
+	if (!node->GetPropVal(wxT("defSize"), &val) || (val != wxT("true") && val != wxT("1")))
 		SetDefaultSize(false);
+	m_displayVideoFrame = !node->GetPropVal(wxT("displayVideoFrame"), &val) || (val != wxT("false") && val != wxT("0"));
+	m_customVideoFrame = node->GetPropVal(wxT("customVideoFrame"), &val) && (val == wxT("true") || val == wxT("1"));
 
 	// paste button
 	wxSvgXmlNode* svgNode = XmlFindNode(node, wxT("svg"));
@@ -814,7 +847,8 @@ bool MenuObject::PutXML(wxSvgXmlNode* node) {
 
 	if (!Init(fileName, x, y))
 		return false;
-
+	
+	// read changeable parameters
 	wxSvgXmlNode* child = node->GetChildren();
 	while (child) {
 		if (child->GetName() == wxT("parameter")) {
@@ -837,6 +871,6 @@ bool MenuObject::PutXML(wxSvgXmlNode* node) {
 		}
 		child = child->GetNext();
 	}
-
+	
 	return true;
 }
