@@ -3,7 +3,7 @@
 // Purpose:     DVD menu properties dialog
 // Author:      Alex Thuering
 // Created:     25.04.2004
-// RCS-ID:      $Id: MenuPropDlg.cpp,v 1.34 2010/10/10 18:39:13 ntalex Exp $
+// RCS-ID:      $Id: MenuPropDlg.cpp,v 1.35 2011/02/13 19:12:14 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -13,15 +13,18 @@
 #include "AVPropDlg.h"
 #include "Config.h"
 #include <wxVillaLib/utils.h>
+#include <wx/file.h>
 #include "rc/preferences.png.h"
 
 enum {
-	AV_BT_ID = 7850,
-	LOOP_CB_ID = 7851
+	ASPECT_CHOICE_ID = 7850,
+	AUDIO_CTRL_ID,
+	LOOP_CB_ID
 };
 
 BEGIN_EVENT_TABLE(MenuPropDlg, wxPropDlg)
-	EVT_BUTTON(AV_BT_ID, MenuPropDlg::OnAV)
+	EVT_CHOICE(ASPECT_CHOICE_ID, MenuPropDlg::OnChangeAspect)
+	EVT_TEXT(AUDIO_CTRL_ID, MenuPropDlg::OnChangeAudio)
 	EVT_CHECKBOX(LOOP_CB_ID, MenuPropDlg::OnLoopCheck)
 END_EVENT_TABLE()
 
@@ -33,6 +36,7 @@ MenuPropDlg::MenuPropDlg(wxWindow *parent, DVD* dvd, int tsi, int pgci):
 	m_pgc = pgcs[pgci];
 	m_vob = m_dvd->GetMenuVob(tsi, pgci);
 	m_menu = m_vob->GetMenu();
+	m_audioFormat = NULL;
 	// set title
 	wxString title = _("Properties") + wxString(wxT(" - "));
 	if (tsi >= 0) {
@@ -56,13 +60,25 @@ void MenuPropDlg::CreatePropPanel(wxSizer* sizer) {
 	formatSizer->AddSpacer(4);
 	formats = DVD::GetAspectRatioLabels();
 	int ar = m_menu->GetAspectRatio() >= 1 ? m_menu->GetAspectRatio() - 1 : 0;
-	formatSizer->Add(AddChoiceProp(formats[ar], formats), 0, wxEXPAND);
-	wxButton* avBt = new wxBitmapButton(this, AV_BT_ID, wxBITMAP_FROM_MEMORY(preferences));
-	avBt->SetHelpText(_("Video properties..."));
-	formatSizer->Add(avBt, 0, wxLEFT, 4);
+	formatSizer->Add(AddChoiceProp(formats[ar], formats, ASPECT_CHOICE_ID), 0, wxEXPAND);
+	m_aspect = (wxChoice*) GetLastControl();
+	formatSizer->AddSpacer(4);
+	wxArrayString wsFormats = m_video->GetWidescreenStrings(_("auto")); 
+	formatSizer->Add(AddChoiceProp(wsFormats[m_video->GetWidescreen()], wsFormats), 0, wxEXPAND);
+	m_wsFormat = (wxChoice*) GetLastControl();
 	formatSizer->AddStretchSpacer(1);
 	sizer->Add(formatSizer, 0, wxEXPAND | wxALL, 6);
+	wxCommandEvent evt;
+	OnChangeAspect(evt);
+	
+	CreateBackgroundGroup(sizer);
+	CreateButtonsGroup(sizer);
+	bool loop = m_vob->GetPause() == 0 && m_pgc->GetPostCommands() == wxT("jump cell 1;");
+	CreateVobGroup(sizer, loop);
+	CreateMenuGroup(sizer, loop);
+}
 
+void MenuPropDlg::CreateBackgroundGroup(wxSizer* sizer) {
 	wxSizer* grpSizer = BeginGroup(sizer, _("Background"));
 	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 2, 4, 16);
 	grid->AddGrowableCol(1);
@@ -70,66 +86,83 @@ void MenuPropDlg::CreatePropPanel(wxSizer* sizer) {
 			_("Image Files ") + wxImage::GetImageExtWildcard()
 			+ wxT("|") + wxString(_("All Files")) + wxT(" (*.*)|*.*"));
 	AddColourProp(grid, _("Colour:"), m_menu->GetBackgroundColour());
-	grpSizer->Add(grid, 0, wxEXPAND|wxTOP|wxLEFT|wxRIGHT, 6);
-	
-	sizer->Add(6,6);
-	
-	grpSizer = BeginGroup(sizer, _("Buttons"));
-	grid = new wxFlexGridSizer(2, 2, 4, 16);
+	grpSizer->Add(grid, 0, wxEXPAND|wxALL, 4);
+	sizer->AddSpacer(6);
+}
+
+void MenuPropDlg::CreateButtonsGroup(wxSizer* sizer) {
+	wxSizer* grpSizer = BeginGroup(sizer, _("Buttons"));
+	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 2, 4, 16);
 	grid->AddGrowableCol(1);
 	wxSizer* timeSizer = AddTextProp(grid, _("Start/End:"), m_menu->GetStartTime(), false, 100, false);
 	AddTextProp(timeSizer, wxT(" - "), m_menu->GetEndTime(), false, 100);
-	grpSizer->Add(grid, 0, wxEXPAND|wxALL, 2);
-	AddCheckProp(grpSizer, _("Remember last selected button"), m_menu->GetRememberLastButton());
-	
-	// VOB properties
-	grpSizer = BeginGroup(sizer, _("Video object"));
-	grid = new wxFlexGridSizer(2, 4, 16);
+	grpSizer->Add(grid, 0, wxEXPAND|wxALL, 4);
+	wxSizer* cbSizer = new wxBoxSizer(wxHORIZONTAL);
+	AddCheckProp(cbSizer, _("Remember last selected button"), m_menu->GetRememberLastButton());
+	grpSizer->Add(cbSizer, 0, wxEXPAND|wxALL, 4);
+	sizer->AddSpacer(6);
+}
+
+void MenuPropDlg::CreateVobGroup(wxSizer* sizer, bool loop) {
+	wxSizer* grpSizer = BeginGroup(sizer, _("Video object"));
+	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 4, 16);
 	grid->AddGrowableCol(1);
+	
 	// audio file
 	wxString audioFile = m_vob->GetAudioFilenames().size() ? m_vob->GetAudioFilenames()[0] : wxT("");
 	wxString audio = wxThumbnails::GetAudioExtWildcard();
 	AddFileProp(grid, _("Audio:"), audioFile, wxFD_OPEN, wxT("..."),
 			_("Audio Files") + wxString::Format(wxT(" (%s)|%s|"), audio.c_str(), audio.c_str())
-			+ _("All Files") + wxString(wxT(" (*.*)|*.*")));
-	// audio language
+			+ _("All Files") + wxString(wxT(" (*.*)|*.*")), AUDIO_CTRL_ID);
+	// audio format
+	Stream* stream = audioFile.length() > 0 ? m_vob->GetStreams()[m_vob->GetStreams().size() - 1] : NULL;
+	int af = stream != NULL ? stream->GetAudioFormat() - 1 : afCOPY - 1;
+	wxArrayString formats = DVD::GetAudioFormatLabels(true);
+	wxSizer* formatSizer = AddChoiceProp(grid, _("Format:"), formats[af], formats, 0, false);
+	m_audioFormat = (wxChoice*) GetLastControl();
+	m_audioFormat->Enable(audioFile.length() > 0);
 	wxArrayString labels = DVD::GetAudioLanguageCodes();
 	if (m_langCodes->GetCount() < 1)
 		m_langCodes->Add(s_config.GetDefAudioLanguage());
-	AddChoiceProp(grid, _("Language:"), (*m_langCodes)[0], labels, 0, false);
-	
+	formatSizer->AddSpacer(4);
+	AddChoiceProp(formatSizer, wxT(""), (*m_langCodes)[0], labels, 0, false);
 	// pause and loop
-	bool loop = m_vob->GetPause() == 0 && m_pgc->GetPostCommands() == wxT("jump cell 1;");
 	wxSizer* pSizer = AddSpinProp(grid, _("Pause:"), m_vob->GetPause(), -1, 254, 100, _("sec"),false);
 	m_pauseCtrl = (wxSpinCtrl*) GetLastControl();
 	m_pauseCtrl->Enable(!loop);
 	pSizer->AddSpacer(4);
 	AddCheckProp(pSizer, _T("Loop"), loop, false, LOOP_CB_ID);
-	grpSizer->Add(grid, 0, wxEXPAND|wxALL, 6);
 	
-	sizer->Add(6,6);
-	
-	grpSizer = BeginGroup(sizer, _("Menu"));
-	grid = new wxFlexGridSizer(2, 2, 4, 16);
+	grpSizer->Add(grid, 0, wxEXPAND|wxALL, 4);
+	sizer->AddSpacer(6);
+}
+
+void MenuPropDlg::CreateMenuGroup(wxSizer* sizer, bool loop) {	
+	wxSizer* grpSizer = BeginGroup(sizer, _("Menu"));
+	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 4, 16);
 	grid->AddGrowableCol(1);
 	
 	const StringSet& entries = m_pgc->GetEntries();
-	if (m_tsi>=0) {
-		AddCheckProp(grid, _("Root"), entries.find(wxT("root")) != entries.end());
-		AddCheckProp(grid, _("Subtitle"), entries.find(wxT("subtitle")) != entries.end());
-		AddCheckProp(grid, _("Chapter"), entries.find(wxT("ptt")) != entries.end());
-		AddCheckProp(grid, _("Angle"), entries.find(wxT("angle")) != entries.end());
-		AddCheckProp(grid, _("Audio"), entries.find(wxT("audio")) != entries.end());
+	AddText(grid, _("Entry"), m_tsi >= 0 ? wxALIGN_TOP : wxALIGN_CENTER_VERTICAL);
+	if (m_tsi >= 0) {
+		wxFlexGridSizer* entryGrid = new wxFlexGridSizer(3, 4, 16);
+		AddCheckProp(entryGrid, _("Root"), entries.find(wxT("root")) != entries.end());
+		AddCheckProp(entryGrid, _("Subtitle"), entries.find(wxT("subtitle")) != entries.end());
+		AddCheckProp(entryGrid, _("Chapter"), entries.find(wxT("ptt")) != entries.end());
+		AddCheckProp(entryGrid, _("Audio"), entries.find(wxT("audio")) != entries.end());
+		AddCheckProp(entryGrid, _("Angle"), entries.find(wxT("angle")) != entries.end());
+		grid->Add(entryGrid);
 	} else
 		AddCheckProp(grid, _("Title"), entries.find(wxT("title")) != entries.end());
-	AddText(grid, wxEmptyString);
 	
+	// Pre and post commans
 	wxArrayString commands = GetCommands();
 	AddComboProp(grid, _("Pre commands:"), m_pgc->GetPreCommands(), commands);
 	AddComboProp(grid, _("Post commands:"), m_pgc->GetPostCommands(), commands);
 	m_postCommandsCtrl = (wxComboBox*) GetLastControl();
 	m_postCommandsCtrl->Enable(!loop);
-	grpSizer->Add(grid, 0, wxEXPAND|wxALL, 6);
+	
+	grpSizer->Add(grid, 0, wxEXPAND|wxALL, 4);
 }
 
 wxArrayString MenuPropDlg::GetCommands() {
@@ -148,9 +181,26 @@ wxArrayString MenuPropDlg::GetCommands() {
 }
 
 bool MenuPropDlg::SetValues() {
+	// validate audio file
+	wxString audioFile = GetString(8);
+	if (audioFile.length() > 0) {
+		if (!wxFile::Exists(audioFile)) {
+			wxLogError(_("File %s doesn't exist."), audioFile.c_str());
+			return false;
+		}
+		wxFfmpegMediaDecoder ffmpeg;
+		if (!ffmpeg.Load(audioFile))
+			return false;
+		if (ffmpeg.GetStreamCount() == 0 || ffmpeg.GetStreamType(0) != stAUDIO) {
+			wxLogError(_("File %s isn't audio file."), audioFile.c_str());
+			return false;
+		}
+	}
+	
 	// validate commands
-	wxString preCommands = GetString(m_tsi >= 0 ? 15 : 11);
-	wxString postCommands = GetString(m_tsi >= 0 ? 16 : 12);
+	int n = m_tsi >= 0 ? 18 : 14;
+	wxString preCommands = GetString(n++);
+	wxString postCommands = GetString(n++);
 	DVDAction action(false, BUTTON_ACTION);
 	action.SetCustom(preCommands);
 	if (preCommands.length() && !action.IsValid(m_dvd, m_tsi, m_pgci, true, wxT(""), true, false)) {
@@ -161,7 +211,7 @@ bool MenuPropDlg::SetValues() {
 		return false;
 	}
 	
-	int n = 0;
+	n = 0;
 	
 	// set video format
 	VideoFormat videoFormat = (VideoFormat) (GetInt(n++) + 2);
@@ -173,6 +223,8 @@ bool MenuPropDlg::SetValues() {
 	if (aspectRatio != m_menu->GetAspectRatio()) {
 		m_dvd->GetPgcArray(m_tsi, true).SetAspectRatio(aspectRatio);
 	}
+	// set WS format
+	m_video->SetWidescreen((WidescreenType) GetInt(n++));
 	
 	m_menu->SetBackground(GetString(n++));
 	if (!GetString(n - 1).length())
@@ -184,11 +236,18 @@ bool MenuPropDlg::SetValues() {
 	m_menu->SetRememberLastButton(GetBool(n++));
 	
 	// VOB properties
-	wxString audioFile = GetString(n++).Strip(wxString::both);
+	audioFile = GetString(n++).Strip(wxString::both);
 	while (m_vob->GetAudioFilenames().GetCount() > 0)
 		m_vob->RemoveAudioFile(0);
-	if (audioFile.length() > 0)
+	if (audioFile.length() > 0) {
 		m_vob->AddAudioFile(audioFile);
+		if (m_vob->GetStreams().size() > 0) {
+			Stream* stream = m_vob->GetStreams()[m_vob->GetStreams().size() - 1];
+			if (stream->GetType() == stAUDIO)
+				stream->SetAudioFormat((AudioFormat) (GetInt(n) + 1));
+		}
+	}
+	n++;
 	m_langCodes->RemoveAt(0);
 	m_langCodes->Insert(GetString(n++), 0);
 	m_vob->SetPause(GetInt(n++));
@@ -225,11 +284,28 @@ bool MenuPropDlg::SetValues() {
 	return true;
 }
 
-void MenuPropDlg::OnAV(wxCommandEvent& event) {
-	AVPropDlg propDlg(this, m_video, true);
-	if (propDlg.ShowModal() == wxID_OK) {
-		wxCommandEvent evt(EVT_COMMAND_DVD_CHANGED, this->GetId());
-		GetEventHandler()->ProcessEvent(evt);
+void MenuPropDlg::OnChangeAspect(wxCommandEvent& event) {
+	AspectRatio aspect = (AspectRatio) (m_aspect->GetSelection() + 1);
+	m_wsFormat->Enable(aspect == ar16_9);
+}
+
+void MenuPropDlg::OnChangeAudio(wxCommandEvent& event) {
+	if (!m_audioFormat)
+		return;
+	wxString audioFile = event.GetString();
+	m_audioFormat->Enable(audioFile.length() > 0);
+	// set audio format
+	wxFfmpegMediaDecoder ffmpeg;
+	if (wxFile::Exists(audioFile) && ffmpeg.Load(audioFile) && ffmpeg.GetStreamCount() > 0
+			&& ffmpeg.GetStreamType(0) == stAUDIO) {
+		AudioFormat af = afCOPY;
+		wxString codecName = ffmpeg.GetCodecName(0);
+		int sampleRate = ffmpeg.GetSampleRate(0);
+		if (codecName != wxT("mp2") && codecName != wxT("ac3") && codecName != wxT("liba52"))
+			af = m_dvd->GetAudioFormat();
+		else if (sampleRate != 48000)
+			af = codecName == wxT("mp2") ? afMP2 : afAC3;
+		m_audioFormat->SetSelection(af - 1);
 	}
 }
 
