@@ -3,7 +3,7 @@
 // Purpose:     Progress of generation/burning dialog
 // Author:      Alex Thuering
 // Created:     14.08.2004
-// RCS-ID:      $Id: ProgressDlg.cpp,v 1.104 2010/10/10 16:01:32 ntalex Exp $
+// RCS-ID:      $Id: ProgressDlg.cpp,v 1.107 2011/03/10 20:07:37 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -217,7 +217,7 @@ protected:
 class ProgressDlgLog: public wxLog {
 public:
 	/**
-	 * Consructor.
+	 * Constructor
 	 */
 	ProgressDlgLog(ProgressDlg* dlg) {
 		this->dlg = dlg;
@@ -418,7 +418,7 @@ int ProgressDlg::GetMenuSubSteps(wxArrayPtrVoid& menuVobs, wxArrayInt& menuWSTyp
 			if (vob->GetMenu()->HasVideoBackground()) {
 				menuSubSteps += 200; // transcode
 			} else if (vob->GetAudioFilenames().size() > 0)
-				menuSubSteps += 300; // generate mpeg(100) + transcode(200)
+				menuSubSteps += 300; // generate mpeg(25+75) + transcode(200)
 			else
 				menuSubSteps += 25; // generate mpeg
 			menuSubSteps += 50; // spumux
@@ -429,7 +429,7 @@ int ProgressDlg::GetMenuSubSteps(wxArrayPtrVoid& menuVobs, wxArrayInt& menuWSTyp
 	return menuSubSteps;
 }
 
-int ProgressDlg::GetTitleSubSteps(wxArrayPtrVoid& titleVobs, DVD* dvd, Cache* cache) {
+int ProgressDlg::GetTitleSubSteps(wxArrayPtrVoid& titleVobs, wxArrayInt& titleAspects, DVD* dvd, Cache* cache) {
 	int titleSubSteps = 0;
 	for (int tsi = 0; tsi < (int) dvd->GetTitlesets().Count(); tsi++) {
 		Titleset* ts = dvd->GetTitlesets()[tsi];
@@ -449,6 +449,7 @@ int ProgressDlg::GetTitleSubSteps(wxArrayPtrVoid& titleVobs, DVD* dvd, Cache* ca
 				}
 				titleSubSteps += 200;
 				titleVobs.Add(vob);
+				titleAspects.Add(ts->GetTitles().GetVideo().GetAspect());
 			}
 		}
 	}
@@ -465,6 +466,8 @@ int ProgressDlg::GetSlideshowSubSteps(wxArrayPtrVoid& slideshowVobs, DVD* dvd, w
 				Vob* vob = pgc->GetVobs()[vobi];
 				if (vob->GetSlideshow()) {
 					slideshowSubSteps += 10 * vob->GetSlideshow()->Count();
+					if (vob->GetAudioFilenames().size() > 0)
+						slideshowSubSteps += 200; // transcode
 					vob->SetTmpFilename(dvdTmpDir
 							+ wxString::Format(_T("title%d-%d-%d.vob"), tsi, pgci, vobi));
 					slideshowVobs.Add(vob);
@@ -475,7 +478,8 @@ int ProgressDlg::GetSlideshowSubSteps(wxArrayPtrVoid& slideshowVobs, DVD* dvd, w
 	return slideshowSubSteps;
 }
 
-int ProgressDlg::GetSubtitleSubSteps(wxArrayPtrVoid& subtitleVobs, DVD* dvd, wxString dvdTmpDir) {
+int ProgressDlg::GetSubtitleSubSteps(wxArrayPtrVoid& subtitleVobs, wxArrayInt& subtitleTsi, DVD* dvd,
+		wxString dvdTmpDir) {
 	int subtitleSubSteps = 0;
 	for (int tsi = 0; tsi < (int) dvd->GetTitlesets().Count(); tsi++) {
 		Titleset* ts = dvd->GetTitlesets()[tsi];
@@ -488,6 +492,7 @@ int ProgressDlg::GetSubtitleSubSteps(wxArrayPtrVoid& subtitleVobs, DVD* dvd, wxS
 					vob->SetTmpFilename(dvdTmpDir
 							+ wxString::Format(_T("title%d-%d-%d.vob"), tsi, pgci, vobi));
 					subtitleVobs.Add(vob);
+					subtitleTsi.Add(tsi);
 				}
 			}
 		}
@@ -525,13 +530,15 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd) {
 	int menuSubSteps = GetMenuSubSteps(menuVobs, menuWSTypes, dvd, dvdTmpDir);
 	// titles
 	wxArrayPtrVoid titleVobs;
-	int titleSubSteps = GetTitleSubSteps(titleVobs, dvd, m_cache);
+	wxArrayInt titleAspects;
+	int titleSubSteps = GetTitleSubSteps(titleVobs, titleAspects, dvd, m_cache);
 	// slideshow
 	wxArrayPtrVoid slideshowVobs;
 	int slideshowSubSteps = GetSlideshowSubSteps(slideshowVobs, dvd, dvdTmpDir);
 	// subtitle
 	wxArrayPtrVoid subtitleVobs;
-	int subtitleSubSteps = GetSubtitleSubSteps(subtitleVobs, dvd, dvdTmpDir);
+	wxArrayInt subtitleTsi;
+	int subtitleSubSteps = GetSubtitleSubSteps(subtitleVobs, subtitleTsi, dvd, dvdTmpDir);
 	// remove unused files from cache
 	m_cache->EndClean();
 
@@ -559,9 +566,9 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd) {
 	
 	// Start generation
 	if (!GenerateMenus(menuVobs, menuWSTypes, menuSubSteps, dvd)
-			|| !Transcode(titleVobs, titleSubSteps, dvd)
+			|| !Transcode(titleVobs, titleAspects, titleSubSteps, dvd)
 			|| !GenerateSlideshow(slideshowVobs, slideshowSubSteps, dvd)
-			|| !MultiplexSubtitles(subtitleVobs, subtitleSubSteps, dvd)
+			|| !MultiplexSubtitles(subtitleVobs, subtitleTsi, subtitleSubSteps, dvd)
 			|| !GenerateDvdFilesystem(dvd, dvdTmpDir, dvdOutDir, tmpDir)
 			|| !Preview(burnDlg, dvdOutDir)
 			|| !CreateIsoImage(burnDlg, dvd, dvdOutDir, tmpDir)
@@ -627,7 +634,7 @@ bool ProgressDlg::GenerateMenus(wxArrayPtrVoid& menuVobs, wxArrayInt& menuWSType
 	return true;
 }
 
-bool ProgressDlg::Transcode(wxArrayPtrVoid& titleVobs, int titleSubSteps, DVD* dvd) {
+bool ProgressDlg::Transcode(wxArrayPtrVoid& titleVobs, wxArrayInt& titleAspects, int titleSubSteps, DVD* dvd) {
 	if (titleVobs.Count() == 0)
 		return true;
 	if (IsCanceled())
@@ -636,10 +643,11 @@ bool ProgressDlg::Transcode(wxArrayPtrVoid& titleVobs, int titleSubSteps, DVD* d
 	SetSubSteps(titleSubSteps);
 	for (int i = 0; i < (int) titleVobs.Count(); i++) {
 		Vob* vob = (Vob*) titleVobs[i];
+		AspectRatio aspect = (AspectRatio) titleAspects[i];
 		wxString cacheFile = m_cache->Add(vob, dvd);
 		AddDetailMsg(wxT("Add file to cache:") + cacheFile);
 		vob->SetTmpFilename(cacheFile);
-		if (!Transcode(vob, dvd->GetVideoBitrate(), s_config.GetUseMplex())) {
+		if (!Transcode(vob, aspect, dvd->GetVideoBitrate(), s_config.GetUseMplex())) {
 			wxRemoveFile(vob->GetTmpFilename());
 			return false;
 		}
@@ -658,15 +666,16 @@ bool ProgressDlg::GenerateSlideshow(wxArrayPtrVoid& slideshowVobs, int slideshow
 	for (int i = 0; i < (int) slideshowVobs.Count(); i++) {
 		AddDetailMsg(wxString::Format(_("Generating slideshow %d of %d"), i + 1, slideshowVobs.Count()));
 		Vob* vob = (Vob*) slideshowVobs[i];
-		if (!GenerateSlideshow(vob->GetSlideshow(), vob->GetTmpFilename(),
-				dvd->GetAudioFormat()))
+		if (!GenerateSlideshow(vob->GetSlideshow(), vob->GetTmpFilename(), dvd->GetAudioFormat(),
+				vob->GetAudioFilenames().size() > 0 ? vob->GetAudioFilenames()[0] : wxT("")))
 			return false;
 	}
 	IncStep();
 	return true;
 }
 
-bool ProgressDlg::MultiplexSubtitles(wxArrayPtrVoid& subtitleVobs, int subtitleSubSteps, DVD* dvd) {
+bool ProgressDlg::MultiplexSubtitles(wxArrayPtrVoid& subtitleVobs, wxArrayInt& subtitleTsi, int subtitleSubSteps,
+		DVD* dvd) {
 	if (subtitleVobs.Count() == 0)
 		return true;
 	if (IsCanceled())
@@ -676,37 +685,24 @@ bool ProgressDlg::MultiplexSubtitles(wxArrayPtrVoid& subtitleVobs, int subtitleS
 	for (int i = 0; i < (int) subtitleVobs.Count(); i++) {
 		AddDetailMsg(wxString::Format(_("Multiplexing subtitles %d of %d"), i + 1, subtitleVobs.Count()));
 		Vob* vob = (Vob*) subtitleVobs[i];
+		AspectRatio aspectRatio = dvd->GetTitlesets()[subtitleTsi[i]]->GetTitles().GetVideo().GetAspect();
 		
 		// Extract Video stream geometry
-		wxSize subtitleSize;
-		const StreamArray streams = vob->GetStreams();
+		wxSize movieSize;
+		VideoFormat videoFormat = vfPAL;
+		Stream* stream = vob->GetVideoStream();
 
-		for (unsigned int stIdx = 0; stIdx < streams.GetCount(); stIdx++) {
-		  Stream* stream = streams[stIdx];
-		  if (stream->GetType() == stVIDEO) {
-		    switch(stream->GetVideoFormat()) {
-		    case vfNTSC:
-		      // Use the specified destination NTSC format
-		      subtitleSize.SetWidth(720);
-		      subtitleSize.SetHeight(480);
-		      break;
-		      
-		    case vfPAL:
-		      // Use the specified destination PAL format
-		      subtitleSize.SetWidth(720);
-		      subtitleSize.SetHeight(574);
-		      break;
-
-		    case vfCOPY:
-		    default:
-		      // Use source format as geometry
-		      subtitleSize = stream->GetSourceVideoSize();
-		      break;
-		    }
-		    break;
-		  }
+		if (stream != NULL) {
+			if (stream->GetVideoFormat() == vfCOPY) {
+				movieSize = stream->GetSourceVideoSize();
+				videoFormat = movieSize.GetHeight() == 480 || movieSize.GetHeight() == 240 ? vfNTSC : vfPAL;
+			} else {
+				movieSize = GetFrameSize(videoFormat);
+				videoFormat = stream->GetVideoFormat();
+			}
 		}
-
+		
+		int subCount = vob->GetSubtitleStreamsCount() - vob->GetSubtitles().Count();
 		for (unsigned int s = 0; s < vob->GetSubtitles().Count(); s++) {
 			wxString vobFile = vob->GetFilename();
 			if (wxFileExists(vob->GetTmpFilename())) {
@@ -719,10 +715,10 @@ bool ProgressDlg::MultiplexSubtitles(wxArrayPtrVoid& subtitleVobs, int subtitleS
 			
 			// Force geometry of subtitle
 			TextSub* textSub = vob->GetSubtitles()[s];
-			textSub->SetMovieWidth(subtitleSize.GetWidth());
-			textSub->SetMovieHeight(subtitleSize.GetHeight());
+			textSub->SetMovieSize(movieSize);
+			textSub->SetAspectRatio(aspectRatio);
 
-			if (!MultiplexSubtitles(vobFile, vob->GetSubtitles()[s], s, vob->GetTmpFilename()))
+			if (!MultiplexSubtitles(vobFile, vob->GetSubtitles()[s], s + subCount, videoFormat, vob->GetTmpFilename()))
 				return false;
 			IncSubStep(vob->GetSize(dvd));
 		}
@@ -732,13 +728,13 @@ bool ProgressDlg::MultiplexSubtitles(wxArrayPtrVoid& subtitleVobs, int subtitleS
 }
 
 bool ProgressDlg::MultiplexSubtitles(const wxString& vobFile, TextSub* textSub, unsigned int streamIdx,
-		const wxString& resultFile) {
+		VideoFormat videoFormat, const wxString& resultFile) {
 	if (IsCanceled())
 		return false;
 	//spumux
 	wxString cmd = s_config.GetSpumuxCmd();
 	wxString spuFile = resultFile + wxT("_spumux.xml");
-	textSub->SaveSpumux(spuFile);
+	textSub->SaveSpumux(spuFile, videoFormat);
 	cmd.Replace(wxT("$FILE_CONF"), spuFile);
 	cmd.Replace(wxT("$STREAM"), wxString::Format(wxT("%d"), streamIdx));
 	if (!Exec(cmd, vobFile, resultFile)) {
@@ -1049,25 +1045,18 @@ bool ProgressDlg::BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir,
 	return true;
 }
 
-bool ProgressDlg::Transcode(Vob* vob, int videoBitrate, bool useMplex) {
+bool ProgressDlg::Transcode(Vob* vob, AspectRatio aspect, int videoBitrate, bool useMplex) {
 	if (IsCanceled())
 		return false;
 	int subStep = GetSubStep();
 	AddDetailMsg(_("Transcode video file: ") + vob->GetFilename());
-	bool disableAudio = vob->GetStreams().GetCount() > 1 + vob->GetAudioFilenames().GetCount()
-		&& vob->GetStreams()[1]->GetType() == stAUDIO && vob->GetStreams()[1]->GetDestinationFormat() == afNONE;
-	wxFfmpegMediaTranscoder transcoder(s_config.GetThreadCount());
-	if (!transcoder.AddInputFile(vob->GetFilename(), false, disableAudio)) {
-		Failed(wxT("Error by transcoding of ") + vob->GetFilename());
-		return false;
-	}
-	for (int i = 0; i < (int)vob->GetAudioFilenames().GetCount(); i++) {
-		if (!transcoder.AddInputFile(vob->GetAudioFilenames()[i])) {
-			Failed(wxT("Error by transcoding of ") + vob->GetAudioFilenames()[i]);
-			return false;
-		}
-	}
 	// set output formats
+	double startTime = vob->GetStartTime();
+	double recordingTime = vob->GetRecordingTime();
+	if (vob->GetSlideshow())
+		recordingTime = vob->GetSlideshow()->GetDuration() * vob->GetSlideshow()->GetCount();
+	bool needEncode = vob->GetAudioFilenames().GetCount() != vob->GetAudioStreamCount()
+			|| startTime != 0 || recordingTime != -1; // some files are not elementary streams
 	VideoFormat videoFormat = vfCOPY;
 	wxArrayInt audioFormats;
 	wxArrayInt subtitleFormats;
@@ -1076,9 +1065,11 @@ bool ProgressDlg::Transcode(Vob* vob, int videoBitrate, bool useMplex) {
 		switch (stream->GetType()) {
 		case stVIDEO:
 			videoFormat = stream->GetVideoFormat();
+			needEncode = needEncode || stream->GetVideoFormat() != vfCOPY;
 			break;
 		case stAUDIO:
 			audioFormats.Add(stream->GetAudioFormat());
+			needEncode = needEncode || stream->GetAudioFormat() != afCOPY;
 			break;
 		case stSUBTITLE:
 			subtitleFormats.Add(stream->GetSubtitleFormat());
@@ -1087,75 +1078,112 @@ bool ProgressDlg::Transcode(Vob* vob, int videoBitrate, bool useMplex) {
 			break;
 		}
 	}
-	transcoder.SetInterlaced(vob->GetInterlaced());
-	transcoder.SetFirstField(vob->GetFirstField());
-	if (vob->GetPad().size() == 4
-			&& vob->GetPad()[0] + vob->GetPad()[1] + vob->GetPad()[2] + vob->GetPad()[3] > 0) {
-		wxLogMessage(wxT("pad: %d %d %d %d"),
-				vob->GetPad()[0], vob->GetPad()[1], vob->GetPad()[2], vob->GetPad()[3]);
-		transcoder.SetPad(vob->GetPad());
-	}
-	if (vob->GetCrop().size() == 4
-				&& vob->GetCrop()[0] + vob->GetCrop()[1] + vob->GetCrop()[2] + vob->GetCrop()[3] > 0) {
-		wxLogMessage(wxT("crop: %d %d %d %d"),
-				vob->GetCrop()[0], vob->GetCrop()[1], vob->GetCrop()[2], vob->GetCrop()[3]);
-		transcoder.SetCrop(vob->GetCrop());
-	}
-	if (useMplex) {
-		wxString videoFile = vob->GetTmpFilename() + wxT(".m2v");
-		if (wxFileExists(videoFile) && !wxRemoveFile(videoFile)) {
-			wxLogError(wxString::Format(_("Can't remove file '%s'"), videoFile.c_str()));
-		}
-		if (!transcoder.SetOutputFile(videoFile, videoFormat, afNONE, sfNONE,
-				videoBitrate, s_config.GetVbr(), s_config.GetAudioBitrate())) {
-			Failed(_("Error transcoding of ") + vob->GetFilename());
+	
+	AddDetailMsg(wxString(wxT("Need encode: ")) + (needEncode ? wxT("true") : wxT("false"))
+			+ wxString(wxT(", use mplex: ")) + (useMplex ? wxT("true") : wxT("false")));
+	
+	// transcode
+	if (useMplex && !needEncode) {
+		if (!Multiplex(vob->GetFilename(), vob->GetAudioFilenames(), vob->GetTmpFilename()))
+			return false;
+	} else {
+		// add input files
+		bool disableAudio = vob->GetStreams().GetCount() > 1 + vob->GetAudioFilenames().GetCount()
+			&& vob->GetStreams()[1]->GetType() == stAUDIO && vob->GetStreams()[1]->GetDestinationFormat() == afNONE;
+		wxFfmpegMediaTranscoder transcoder(s_config.GetThreadCount());
+		if (!transcoder.AddInputFile(vob->GetFilename(), false, disableAudio)) {
+			Failed(wxT("Error by transcoding of ") + vob->GetFilename());
 			return false;
 		}
-		wxArrayString audioFiles;
-		for (unsigned int audioIdx = 0; audioIdx < audioFormats.GetCount(); audioIdx++) {
-			if (audioFormats[audioIdx] == afNONE)
-				continue;
-			wxString audioFile = vob->GetTmpFilename() + wxString::Format(wxT(".audio%d"), audioIdx);
-			audioFiles.Add(audioFile);
-			if (wxFileExists(audioFile) && !wxRemoveFile(audioFile)) {
-				wxLogError(wxString::Format(_("Can't remove file '%s'"), audioFile.c_str()));
+		for (int i = 0; i < (int)vob->GetAudioFilenames().GetCount(); i++) {
+			if (!transcoder.AddInputFile(vob->GetAudioFilenames()[i])) {
+				Failed(wxT("Error by transcoding of ") + vob->GetAudioFilenames()[i]);
+				return false;
 			}
-			if (!transcoder.SetOutputFile(audioFile, vfNONE, (AudioFormat) audioFormats[audioIdx], sfNONE,
+		}
+		
+		// set output options
+		transcoder.SetInterlaced(vob->GetInterlaced());
+		transcoder.SetFirstField(vob->GetFirstField());
+		if (vob->GetKeepAspectRatio()) {
+			vob->UpdatePad(aspect);
+		}
+		if (vob->GetPad()[0] + vob->GetPad()[1] + vob->GetPad()[2] + vob->GetPad()[3] > 0) {
+			wxLogMessage(wxT("pad: %d %d %d %d"),
+					vob->GetPad()[0], vob->GetPad()[1], vob->GetPad()[2], vob->GetPad()[3]);
+			transcoder.SetPad(vob->GetPad());
+		}
+		if (vob->GetCrop()[0] + vob->GetCrop()[1] + vob->GetCrop()[2] + vob->GetCrop()[3] > 0) {
+			wxLogMessage(wxT("crop: %d %d %d %d"),
+					vob->GetCrop()[0], vob->GetCrop()[1], vob->GetCrop()[2], vob->GetCrop()[3]);
+			transcoder.SetCrop(vob->GetCrop());
+		}
+		
+		if (startTime != 0) {
+			wxLogMessage(wxT("startTime: %f"), startTime);
+			transcoder.SetStartTime(startTime);
+		}
+		if (recordingTime > 0) {
+			wxLogMessage(wxT("recordingTime: %f"), recordingTime);
+			transcoder.SetRecordingTime(recordingTime);
+		}
+		
+		if (!useMplex) {
+			if (!transcoder.SetOutputFile(vob->GetTmpFilename(), videoFormat, audioFormats, subtitleFormats,
+					videoBitrate, s_config.GetVbr(), s_config.GetAudioBitrate())
+					|| !transcoder.Run(m_cancel)) {
+				wxRemoveFile(vob->GetTmpFilename());
+				Failed(_("Error transcoding of ") + vob->GetFilename());
+				return false;
+			}
+		} else {
+			wxString videoFile = vob->GetTmpFilename() + wxT(".m2v");
+			if (wxFileExists(videoFile) && !wxRemoveFile(videoFile)) {
+				wxLogError(wxString::Format(_("Can't remove file '%s'"), videoFile.c_str()));
+			}
+			if (!transcoder.SetOutputFile(videoFile, videoFormat, afNONE, sfNONE,
 					videoBitrate, s_config.GetVbr(), s_config.GetAudioBitrate())) {
 				Failed(_("Error transcoding of ") + vob->GetFilename());
 				return false;
 			}
-		}
-		if (!transcoder.Run(m_cancel)) {
-			wxRemoveFile(videoFile);
-			for (unsigned int audioIdx = 0; audioIdx < audioFiles.GetCount(); audioIdx++)
-				wxRemoveFile(audioFiles[audioIdx]);
-			Failed(_("Error transcoding of ") + vob->GetFilename());
-			return false;
-		}
-		SetSubStep(subStep+150);
-		if (!Multiplex(videoFile, audioFiles, vob->GetTmpFilename())) {
-			wxRemoveFile(videoFile);
-			for (unsigned int audioIdx = 0; audioIdx < audioFiles.GetCount(); audioIdx++)
-				wxRemoveFile(audioFiles[audioIdx]);
-			return false;
-		}
-		// remove temp files
-		if (s_config.GetRemoveTempFiles()) {
-			if (!DeleteFile(videoFile))
-				return false;
-			for (unsigned int audioIdx = 0; audioIdx < audioFiles.GetCount(); audioIdx++) {
-				if (!DeleteFile(audioFiles[audioIdx]))
+			wxArrayString audioFiles;
+			for (unsigned int audioIdx = 0; audioIdx < audioFormats.GetCount(); audioIdx++) {
+				if (audioFormats[audioIdx] == afNONE)
+					continue;
+				wxString audioFile = vob->GetTmpFilename() + wxString::Format(wxT(".audio%d"), audioIdx);
+				audioFiles.Add(audioFile);
+				if (wxFileExists(audioFile) && !wxRemoveFile(audioFile)) {
+					wxLogError(wxString::Format(_("Can't remove file '%s'"), audioFile.c_str()));
+				}
+				if (!transcoder.SetOutputFile(audioFile, vfNONE, (AudioFormat) audioFormats[audioIdx], sfNONE,
+						videoBitrate, s_config.GetVbr(), s_config.GetAudioBitrate())) {
+					Failed(_("Error transcoding of ") + vob->GetFilename());
 					return false;
+				}
+			}
+			if (!transcoder.Run(m_cancel)) {
+				wxRemoveFile(videoFile);
+				for (unsigned int audioIdx = 0; audioIdx < audioFiles.GetCount(); audioIdx++)
+					wxRemoveFile(audioFiles[audioIdx]);
+				Failed(_("Error transcoding of ") + vob->GetFilename());
+				return false;
+			}
+			SetSubStep(subStep+150);
+			if (!Multiplex(videoFile, audioFiles, vob->GetTmpFilename())) {
+				wxRemoveFile(videoFile);
+				for (unsigned int audioIdx = 0; audioIdx < audioFiles.GetCount(); audioIdx++)
+					wxRemoveFile(audioFiles[audioIdx]);
+				return false;
+			}
+			// remove temp files
+			if (s_config.GetRemoveTempFiles()) {
+				if (!DeleteFile(videoFile))
+					return false;
+				for (unsigned int audioIdx = 0; audioIdx < audioFiles.GetCount(); audioIdx++) {
+					if (!DeleteFile(audioFiles[audioIdx]))
+						return false;
+				}
 			}
-		}
-	} else {
-		if (!transcoder.SetOutputFile(vob->GetTmpFilename(), videoFormat, audioFormats, subtitleFormats,
-				videoBitrate, s_config.GetVbr(), s_config.GetAudioBitrate())
-				|| !transcoder.Run(m_cancel)) {
-			wxRemoveFile(vob->GetTmpFilename());
-			Failed(_("Error transcoding of ") + vob->GetFilename());
-			return false;
 		}
 	}
 	SetSubStep(subStep+200);
@@ -1235,8 +1263,9 @@ bool ProgressDlg::GenerateMenu(Menu* menu, WidescreenType widescreenType, const
 					}
 				}
 			}
+			menuVob.SetKeepAspectRatio(false);
 			menuVob.SetTmpFilename(mpegFile);
-			if (!Transcode(&menuVob, videoBitrate, s_config.GetUseMplexForMenus()))
+			if (!Transcode(&menuVob, menu->GetAspectRatio(), videoBitrate, s_config.GetUseMplexForMenus()))
 				return false;
 		} else
 			mpegFile = menu->GetBackground();
@@ -1276,8 +1305,8 @@ bool ProgressDlg::GenerateMenu(Menu* menu, WidescreenType widescreenType, const
 			menuVob.AddAudioFile(audioFile);
 			for (unsigned int i=0; i<menuVob.GetStreams().GetCount(); i++) {
 				Stream* stream = menuVob.GetStreams()[i];
-				if (stream->GetType() == stAUDIO) {
-					// set destination format (default COPY) if codec is not mp2/ac3 or sample rate != 48 kHz
+				if (stream->GetType() == stAUDIO && stream->GetAudioFormat() == afCOPY) {
+					// set destination format if it set to COPY and codec is not mp2/ac3 or sample rate != 48 kHz
 					if (stream->GetSourceCodecName() != wxT("mp2")
 							&& stream->GetSourceCodecName() != wxT("ac3")
 							&& stream->GetSourceCodecName() != wxT("liba52")) {
@@ -1288,7 +1317,7 @@ bool ProgressDlg::GenerateMenu(Menu* menu, WidescreenType widescreenType, const
 				}
 			}
 			menuVob.SetTmpFilename(mpegFile);
-			if (!Transcode(&menuVob, videoBitrate, s_config.GetUseMplexForMenus()))
+			if (!Transcode(&menuVob, menu->GetAspectRatio(), videoBitrate, s_config.GetUseMplexForMenus()))
 				return false;
 			if (s_config.GetRemoveTempFiles())
 				if (!DeleteFile(m2vFile))
@@ -1342,13 +1371,17 @@ bool ProgressDlg::GenerateMenu(Menu* menu, WidescreenType widescreenType, const
 	return true;
 }
 
-bool ProgressDlg::GenerateSlideshow(Slideshow* slideshow, const wxString& vobFile, AudioFormat audioFormat) {
+bool ProgressDlg::GenerateSlideshow(Slideshow* slideshow, const wxString& vobFile, AudioFormat audioFormat,
+		wxString audioFile) {
 	if (IsCanceled())
 		return false;
 	AddDetailMsg(_("Generating slideshow"));
+	
+	wxString m2vFile = vobFile + wxT("_video.m2v");
 
 	wxFfmpegMediaEncoder ffmpeg(s_config.GetThreadCount());
-	if (!ffmpeg.BeginEncode(vobFile, slideshow->GetVideoFormat(), audioFormat, slideshow->GetAspectRatio(),
+	if (!ffmpeg.BeginEncode(audioFile.length() ? m2vFile : vobFile, slideshow->GetVideoFormat(),
+			audioFile.length() ? afNONE : audioFormat, slideshow->GetAspectRatio(),
 			s_config.GetSlideshowVideoBitrate())) {
 		Failed(_("Error creation of slideshow"));
 		return false;
@@ -1365,6 +1398,36 @@ bool ProgressDlg::GenerateSlideshow(Slideshow* slideshow, const wxString& vobFil
 		IncSubStep(10);
 	}
 	ffmpeg.EndEncode();
+	
+	if (audioFile.length()) {
+		// mplex (and optionally transcode audio)
+		if (IsCanceled())
+			return false;
+		AddDetailMsg(_("Multiplexing audio and video"));
+		Vob slideShowVob;
+		slideShowVob.SetFilename(m2vFile);
+		slideShowVob.AddAudioFile(audioFile);
+		for (unsigned int i=0; i<slideShowVob.GetStreams().GetCount(); i++) {
+			Stream* stream = slideShowVob.GetStreams()[i];
+			if (stream->GetType() == stAUDIO && stream->GetAudioFormat() == afCOPY) {
+				// set destination format if it set to COPY and codec is not mp2/ac3 or sample rate != 48 kHz
+				if (stream->GetSourceCodecName() != wxT("mp2")
+						&& stream->GetSourceCodecName() != wxT("ac3")
+						&& stream->GetSourceCodecName() != wxT("liba52")) {
+					stream->SetDestinationFormat(audioFormat);
+				} else if (stream->GetSourceSampleRate() != 48000) {
+					stream->SetDestinationFormat(stream->GetSourceCodecName() == wxT("mp2") ? afMP2 : afAC3);
+				}
+			}
+		}
+		slideShowVob.SetTmpFilename(vobFile);
+		if (!Transcode(&slideShowVob, slideshow->GetAspectRatio(), s_config.GetSlideshowVideoBitrate(),
+				s_config.GetUseMplexForMenus()))
+			return false;
+		if (s_config.GetRemoveTempFiles())
+			if (!DeleteFile(m2vFile))
+				return false;
+	}
 
 	wxYield();
 	return true;
