@@ -3,7 +3,7 @@
 // Purpose:     Settings dialog
 // Author:      Alex Thuering
 // Created:     27.03.2004
-// RCS-ID:      $Id: SettingsDlg.cpp,v 1.54 2010/05/10 20:42:34 ntalex Exp $
+// RCS-ID:      $Id: SettingsDlg.cpp,v 1.58 2010/09/16 18:29:26 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -127,8 +127,11 @@ void SettingsDlg::CreatePropPanel(wxSizer* sizer) {
 		if (button == s_config.GetDefButton(def))
 			sel = m_buttons.GetCount() - 1;
 	}
-	AddChoiceProp(interfaceGrid, _("Default button:"), labels[sel], labels, 0);
+	wxSizer* dbtSizer = AddChoiceProp(interfaceGrid, _("Default button:"), labels[sel], labels, 0, false);
 	wxControl* defButtonChoice = GetLastControl();
+	if (dbtSizer)
+		dbtSizer->AddSpacer(4);
+	AddCheckProp(dbtSizer, _("Accept invalid actions"), s_config.GetAcceptInvalidActions(def));
 
 	// File Browser start directory
 	wxString fbDir = s_config.GetFileBrowserDir();
@@ -181,7 +184,11 @@ void SettingsDlg::CreatePropPanel(wxSizer* sizer) {
 	}
 
 	// ----------------- Core Tab -------------------- //
-	AddSpinProp(coreGrid, _("Frame count of menu:"), s_config.GetMenuFrameCount(def), 1, 9999, 80);
+	wxSizer* menuSizer = AddSpinProp(coreGrid, _("Frame count of menu:"), s_config.GetMenuFrameCount(def), 1, 9999, 80,
+			wxT(""), false);
+	if (menuSizer)
+		menuSizer->AddSpacer(6);
+	AddCheckProp(menuSizer, _("Draw buttons on background"), s_config.GetDrawButtonsOnBackground(def));
 	AddSpinProp(coreGrid, _("Menu video bitrate:"), s_config.GetMenuVideoBitrate(def), 1, 9999999, 80);
 	AddSpinProp(coreGrid, _("Slideshow video bitrate:"), s_config.GetSlideshowVideoBitrate(def), 1, 9999999, 80);
 	AddSpinProp(coreGrid, _("Audio bitrate:"), s_config.GetAudioBitrate(def), 1, 9999999, 80);
@@ -202,7 +209,8 @@ void SettingsDlg::CreatePropPanel(wxSizer* sizer) {
 
 	wxSizer* grpSizer = BeginGroup(debugSizer, _("Debug"));
 	AddCheckProp(grpSizer, _("Don't remove temp files"), !s_config.GetRemoveTempFiles(def));
-	AddCheckProp(grpSizer, _("Don't use mplex"), !s_config.GetUseMplex(def));
+	AddCheckProp(grpSizer, _("Use mplex"), s_config.GetUseMplex(def));
+	AddCheckProp(grpSizer, _("Use mplex for menus"), s_config.GetUseMplexForMenus(def));
 }
 
 bool SettingsDlg::SetValues() {
@@ -221,12 +229,14 @@ bool SettingsDlg::SetValues() {
 	s_config.SetDefAudioLanguage(GetString(i++));
 	s_config.SetDefChapterLength(GetInt(i++));
 	s_config.SetDefButton(m_buttons[GetInt(i++)]);
+	s_config.SetAcceptInvalidActions(GetBool(i++));
 	i++;
 	s_config.SetFileBrowserDir(GetString(i++));
 	s_config.SetUndoHistorySize(GetInt(i++));
 	
 	// system core settings
 	s_config.SetMenuFrameCount(GetInt(i++));
+	s_config.SetDrawButtonsOnBackground(GetBool(i++));
 	s_config.SetMenuVideoBitrate(GetInt(i++));
 	s_config.SetSlideshowVideoBitrate(GetInt(i++));
 	s_config.SetAudioBitrate(GetInt(i++));
@@ -239,7 +249,8 @@ bool SettingsDlg::SetValues() {
 	s_config.SetAddECCCmd(GetString(i++));
 	s_config.SetFormatCmd(GetString(i++));
 	s_config.SetRemoveTempFiles(!GetBool(i++));
-	s_config.SetUseMplex(!GetBool(i++));
+	s_config.SetUseMplex(GetBool(i++));
+	s_config.SetUseMplexForMenus(GetBool(i++));
 	return true;
 }
 
