@@ -3,7 +3,7 @@
 // Purpose:     The subtitle properties dialog
 // Author:      Alex Thuering
 // Created:     24.02.2010
-// RCS-ID:      $Id: SubtitlePropDlg.cpp,v 1.4 2010/06/04 07:27:17 ntalex Exp $
+// RCS-ID:      $Id: SubtitlePropDlg.cpp,v 1.7 2011/02/22 18:51:04 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -11,6 +11,7 @@
 #include "SubtitlePropDlg.h"
 #include "Utils.h"
 #include <wx/filename.h>
+#include <wx/colordlg.h>
 #include <wxVillaLib/utils.h>
 
 #include <fontconfig/fontconfig.h>
@@ -21,6 +22,11 @@ FontMap SubtitlePropDlg::s_fonts;
 // begin wxGlade: ::extracode
 
 // end wxGlade
+enum {
+	FILL_BT_ID = 2000,
+	OUTLINE_BT_ID,
+	SHADOW_BT_ID
+};
 
 BEGIN_EVENT_TABLE(SubtitlePropDlg, wxDialog)
     // begin wxGlade: SubtitlePropDlg::event_table
@@ -28,6 +34,9 @@ BEGIN_EVENT_TABLE(SubtitlePropDlg, wxDialog)
     EVT_LISTBOX(FONT_STYLE_BOX_ID , SubtitlePropDlg::OnFontStyleSelected)
     EVT_SPINCTRL(FONT_SIZE_SPIN_ID , SubtitlePropDlg::OnFontSizeChanged)
     EVT_LISTBOX(FONT_SIZE_BOX_ID , SubtitlePropDlg::OnFontSizeSelected)
+    EVT_BUTTON(FILL_BT_ID, SubtitlePropDlg::OnSelectFillColor)
+    EVT_BUTTON(OUTLINE_BT_ID, SubtitlePropDlg::OnSelectOutlineColor)
+    EVT_BUTTON(SHADOW_BT_ID, SubtitlePropDlg::OnSelectShadowColor)
     // end wxGlade
 END_EVENT_TABLE()
 
@@ -58,6 +67,20 @@ SubtitlePropDlg::SubtitlePropDlg(wxWindow* parent, TextSub* textsub, const wxStr
     m_fontSizeBox = new wxListBox(this, FONT_SIZE_BOX_ID , wxDefaultPosition, wxDefaultSize, 0, m_fontSizeBox_choices, wxLB_SINGLE);
     m_previewLabel = new wxStaticText(this, wxID_ANY, _("Preview:"));
     m_previewText = new wxTextCtrl(this, wxID_ANY, _("ABCabcXYZxyz"));
+    m_fillLabel = new wxStaticText(this, wxID_ANY, _("Fill:"));
+    m_fillPanel = new wxPanel(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL);
+    m_fillBt = new wxButton(this, FILL_BT_ID, _("..."));
+    m_outlineLabel = new wxStaticText(this, wxID_ANY, _("Outline:"));
+    m_outlinePanel = new wxPanel(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL);
+    m_outlineBt = new wxButton(this, OUTLINE_BT_ID, _("..."));
+    m_thicknessLabel = new wxStaticText(this, wxID_ANY, _("Thickness:"));
+    m_thicknessCtrl = new wxTextCtrl(this, wxID_ANY, wxEmptyString);
+    m_shadowLabel = new wxStaticText(this, wxID_ANY, _("Shadow:"));
+    m_shadowPanel = new wxPanel(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL);
+    m_shadowBt = new wxButton(this, SHADOW_BT_ID, _("..."));
+    m_offsetLabel = new wxStaticText(this, wxID_ANY, _("Offset:"));
+    m_offsetXCtrl = new wxSpinCtrl(this, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 100);
+    m_offsetYCtrl = new wxSpinCtrl(this, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 100);
     m_alignRadioTL = new wxRadioButton(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
     m_alignRadioTC = new wxRadioButton(this, wxID_ANY, wxEmptyString);
     m_alignRadioTR = new wxRadioButton(this, wxID_ANY, wxEmptyString);
@@ -86,6 +109,15 @@ void SubtitlePropDlg::set_properties() {
     m_fontStyleBox->SetMinSize(wxSize(140, 40));
     m_fontSizeSpin->SetMinSize(wxSize(48, -1));
     m_fontSizeBox->SetMinSize(wxSize(-1, 40));
+    m_fillPanel->SetMinSize(wxSize(20, 20));
+    m_fillBt->SetMinSize(wxSize(24, 24));
+    m_outlinePanel->SetMinSize(wxSize(20, 20));
+    m_outlineBt->SetMinSize(wxSize(24, 24));
+    m_thicknessCtrl->SetMinSize(wxSize(80, -1));
+    m_shadowPanel->SetMinSize(wxSize(20, 20));
+    m_shadowBt->SetMinSize(wxSize(24, 24));
+    m_offsetXCtrl->SetMinSize(wxSize(60, -1));
+    m_offsetYCtrl->SetMinSize(wxSize(60, -1));
     m_alignRadioBC->SetValue(1);
     m_marginTop->SetMinSize(wxSize(48, -1));
     m_marginLeft->SetMinSize(wxSize(48, -1));
@@ -127,6 +159,16 @@ void SubtitlePropDlg::set_properties() {
 	m_fontSizeSpin->SetValue((int)m_textsub->GetFontSize());
 	wxSpinEvent spnEvt;
 	OnFontSizeChanged(spnEvt);
+	
+	// colours
+	SetFillColour(m_textsub->GetFillColour());
+	SetOutlineColour(m_textsub->GetOutlineColour());
+	SetShadowColour(m_textsub->GetShadowColour());
+	m_thicknessCtrl->SetValue(wxString::Format(wxT("%0.1f"), m_textsub->GetOutlineThickness()));
+	m_offsetXCtrl->SetValue(m_textsub->GetShadowOffset().x);
+	m_offsetYCtrl->SetValue(m_textsub->GetShadowOffset().y);
+	
+	// align
 	m_alignRadioTL->SetValue(m_textsub->GetAlignment() == (wxALIGN_TOP | wxALIGN_LEFT));
 	m_alignRadioTC->SetValue(m_textsub->GetAlignment() == (wxALIGN_TOP | wxALIGN_CENTER_HORIZONTAL));
 	m_alignRadioTR->SetValue(m_textsub->GetAlignment() == (wxALIGN_TOP | wxALIGN_RIGHT));
@@ -146,12 +188,15 @@ void SubtitlePropDlg::do_layout() {
     // begin wxGlade: SubtitlePropDlg::do_layout
     wxBoxSizer* mainSizer = new wxBoxSizer(wxVERTICAL);
     wxBoxSizer* buttonSizer = new wxBoxSizer(wxHORIZONTAL);
-    wxFlexGridSizer* propSizer = new wxFlexGridSizer(4, 2, 4, 2);
+    wxFlexGridSizer* propSizer = new wxFlexGridSizer(7, 2, 4, 2);
     wxBoxSizer* propSizer2 = new wxBoxSizer(wxHORIZONTAL);
     wxStaticBoxSizer* boxSizer2 = new wxStaticBoxSizer(boxSizer2_staticbox, wxHORIZONTAL);
     wxFlexGridSizer* marginSizer = new wxFlexGridSizer(3, 3, 4, 4);
     wxStaticBoxSizer* boxSizer1 = new wxStaticBoxSizer(boxSizer1_staticbox, wxHORIZONTAL);
     wxGridSizer* alignSizer = new wxGridSizer(3, 3, 12, 16);
+    wxBoxSizer* shadowSizer = new wxBoxSizer(wxHORIZONTAL);
+    wxBoxSizer* outlineSizer = new wxBoxSizer(wxHORIZONTAL);
+    wxBoxSizer* fillSizer = new wxBoxSizer(wxHORIZONTAL);
     wxBoxSizer* fontSizer = new wxBoxSizer(wxHORIZONTAL);
     wxBoxSizer* fSizer1 = new wxBoxSizer(wxVERTICAL);
     wxBoxSizer* fSizer2 = new wxBoxSizer(wxHORIZONTAL);
@@ -175,6 +220,26 @@ void SubtitlePropDlg::do_layout() {
     fSizer1->Add(m_previewText, 0, wxRIGHT|wxTOP|wxEXPAND, 2);
     fontSizer->Add(fSizer1, 1, wxEXPAND, 0);
     propSizer->Add(fontSizer, 0, wxEXPAND, 0);
+    propSizer->Add(m_fillLabel, 0, wxALIGN_CENTER_VERTICAL, 0);
+    fillSizer->Add(m_fillPanel, 0, wxRIGHT|wxALIGN_CENTER_VERTICAL, 4);
+    fillSizer->Add(m_fillBt, 0, 0, 0);
+    fillSizer->Add(20, 20, 1, wxEXPAND, 0);
+    propSizer->Add(fillSizer, 1, wxEXPAND, 0);
+    propSizer->Add(m_outlineLabel, 0, wxALIGN_CENTER_VERTICAL, 0);
+    outlineSizer->Add(m_outlinePanel, 0, wxRIGHT|wxALIGN_CENTER_VERTICAL, 4);
+    outlineSizer->Add(m_outlineBt, 0, wxRIGHT, 4);
+    outlineSizer->Add(m_thicknessLabel, 0, wxRIGHT|wxALIGN_CENTER_VERTICAL, 2);
+    outlineSizer->Add(m_thicknessCtrl, 0, 0, 0);
+    outlineSizer->Add(16, 16, 1, wxEXPAND, 0);
+    propSizer->Add(outlineSizer, 1, wxEXPAND, 0);
+    propSizer->Add(m_shadowLabel, 0, wxALIGN_CENTER_VERTICAL, 0);
+    shadowSizer->Add(m_shadowPanel, 0, wxRIGHT|wxALIGN_CENTER_VERTICAL, 4);
+    shadowSizer->Add(m_shadowBt, 0, wxRIGHT, 4);
+    shadowSizer->Add(m_offsetLabel, 0, wxRIGHT|wxALIGN_CENTER_VERTICAL, 2);
+    shadowSizer->Add(m_offsetXCtrl, 0, wxRIGHT, 2);
+    shadowSizer->Add(m_offsetYCtrl, 0, 0, 0);
+    shadowSizer->Add(20, 20, 1, wxEXPAND, 0);
+    propSizer->Add(shadowSizer, 1, wxEXPAND, 0);
     propSizer->Add(20, 20, 0, wxEXPAND, 0);
     alignSizer->Add(m_alignRadioTL, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
     alignSizer->Add(m_alignRadioTC, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 0);
@@ -353,6 +418,17 @@ int SubtitlePropDlg::ShowModal() {
 		m_textsub->SetFontFamily(m_fontFamilyBox->GetStringSelection());
 		m_textsub->SetFontStyle(m_fontStyleBox->GetStringSelection());
 		m_textsub->SetFontSize(m_fontSizeSpin->GetValue());
+		
+		// colours
+		m_textsub->SetFillColour(GetFillColour());
+		m_textsub->SetOutlineColour(GetOutlineColour());
+		m_textsub->SetShadowColour(GetShadowColour());
+		double dval;
+		if (m_thicknessCtrl->GetValue().ToDouble(&dval))
+			m_textsub->SetOutlineThickness(dval);
+		m_textsub->SetShadowOffset(wxPoint(m_offsetXCtrl->GetValue(), m_offsetYCtrl->GetValue()));
+		
+		// alignment
 		if (m_alignRadioTL->GetValue())
 			m_textsub->SetAlignment((wxAlignment) (wxALIGN_TOP | wxALIGN_LEFT));
 		else if (m_alignRadioTC->GetValue())
@@ -369,6 +445,8 @@ int SubtitlePropDlg::ShowModal() {
 			m_textsub->SetAlignment((wxAlignment) (wxALIGN_BOTTOM | wxALIGN_CENTER_HORIZONTAL));
 		else if (m_alignRadioBR->GetValue())
 			m_textsub->SetAlignment((wxAlignment) (wxALIGN_BOTTOM | wxALIGN_RIGHT));
+		
+		// margin
 		m_textsub->SetLeftMargin(m_marginLeft->GetValue());
 		m_textsub->SetRightMargin(m_marginRight->GetValue());
 		m_textsub->SetTopMargin(m_marginTop->GetValue());
@@ -383,3 +461,27 @@ int SubtitlePropDlg::ShowModal() {
 wxString SubtitlePropDlg::GetLangCode() {
 	return DVD::GetLanguageMap()[m_langChoice->GetStringSelection()];
 }
+
+void SubtitlePropDlg::OnSelectFillColor(wxCommandEvent &event) {
+	wxColourData data;
+	data.SetColour(GetFillColour());
+	wxColourDialog dialog(this, &data);
+	if (dialog.ShowModal() == wxID_OK)
+		SetFillColour(dialog.GetColourData().GetColour());
+}
+
+void SubtitlePropDlg::OnSelectOutlineColor(wxCommandEvent &event) {
+	wxColourData data;
+	data.SetColour(GetOutlineColour());
+	wxColourDialog dialog(this, &data);
+	if (dialog.ShowModal() == wxID_OK)
+		SetOutlineColour(dialog.GetColourData().GetColour());
+}
+
+void SubtitlePropDlg::OnSelectShadowColor(wxCommandEvent &event) {
+	wxColourData data;
+	data.SetColour(GetShadowColour());
+	wxColourDialog dialog(this, &data);
+	if (dialog.ShowModal() == wxID_OK)
+		SetShadowColour(dialog.GetColourData().GetColour());
+}
