@@ -3,7 +3,7 @@
 // Purpose:     The subtitle properties dialog
 // Author:      Alex Thuering
 // Created:     24.02.2010
-// RCS-ID:      $Id: SubtitlePropDlg.h,v 1.2 2010/06/04 07:27:17 ntalex Exp $
+// RCS-ID:      $Id: SubtitlePropDlg.h,v 1.3 2011/02/20 18:55:09 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -76,6 +76,20 @@ private:
     wxListBox* m_fontSizeBox;
     wxStaticText* m_previewLabel;
     wxTextCtrl* m_previewText;
+    wxStaticText* m_fillLabel;
+    wxPanel* m_fillPanel;
+    wxButton* m_fillBt;
+    wxStaticText* m_outlineLabel;
+    wxPanel* m_outlinePanel;
+    wxButton* m_outlineBt;
+    wxStaticText* m_thicknessLabel;
+    wxTextCtrl* m_thicknessCtrl;
+    wxStaticText* m_shadowLabel;
+    wxPanel* m_shadowPanel;
+    wxButton* m_shadowBt;
+    wxStaticText* m_offsetLabel;
+    wxSpinCtrl* m_offsetXCtrl;
+    wxSpinCtrl* m_offsetYCtrl;
     wxRadioButton* m_alignRadioTL;
     wxRadioButton* m_alignRadioTC;
     wxRadioButton* m_alignRadioTR;
@@ -99,7 +113,19 @@ private:
     virtual void OnFontStyleSelected(wxCommandEvent &event); // wxGlade: <event_handler>
     virtual void OnFontSizeChanged(wxSpinEvent &event); // wxGlade: <event_handler>
     virtual void OnFontSizeSelected(wxCommandEvent &event); // wxGlade: <event_handler>
-    void OnFontChanged();
+    virtual void OnSelectFillColor(wxCommandEvent &event); // wxGlade: <event_handler>
+    virtual void OnSelectOutlineColor(wxCommandEvent &event); // wxGlade: <event_handler>
+    virtual void OnSelectShadowColor(wxCommandEvent &event); // wxGlade: <event_handler>
+    virtual void OnFontChanged();
+    
+    wxColour GetFillColour() { return m_fillPanel->GetBackgroundColour(); }
+    void SetFillColour(const wxColour& value) { m_fillPanel->SetBackgroundColour(value); m_fillPanel->Refresh(); }
+    
+    wxColour GetOutlineColour() { return m_outlinePanel->GetBackgroundColour(); }
+    void SetOutlineColour(const wxColour& value) { m_outlinePanel->SetBackgroundColour(value); m_outlinePanel->Refresh(); }
+    
+    wxColour GetShadowColour() { return m_shadowPanel->GetBackgroundColour(); }
+    void SetShadowColour(const wxColour& value) { m_shadowPanel->SetBackgroundColour(value); m_shadowPanel->Refresh(); }
     
 protected:
     DECLARE_EVENT_TABLE()
