@@ -3,7 +3,7 @@
 // Purpose:     DVD title properties dialog
 // Author:      Alex Thuering
 // Created:     31.01.2004
-// RCS-ID:      $Id: TitlePropDlg.h,v 1.17 2010/10/10 18:39:55 ntalex Exp $
+// RCS-ID:      $Id: TitlePropDlg.h,v 1.18 2011/02/13 19:12:56 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -13,6 +13,7 @@
 
 #include <wx/wx.h>
 #include <wxVillaLib/PropDlg.h>
+#include <wxVillaLib/Thumbnails.h>
 #include "DVD.h"
 #include "VobListBox.h"
 
@@ -29,12 +30,19 @@ private:
 	Pgc* m_pgc;
 	Vob* m_vob;
 	int m_vobi;
+	wxArrayString* m_langCodes;
 	VobListBox* m_vobListBox;
 	wxCheckBox* m_doNotTranscodeCheckBox;
 	wxButton* m_addBt;
 	wxButton* m_delBt;
 	wxButton* m_propBt;
+	wxThumbnails* m_slideShowBox;
+	wxChoice* m_audioFormat;
 	void CreatePropPanel(wxSizer* sizer);
+	void CreateVobGroup(wxSizer* sizer);
+	void CreateSlideshowGroup(wxSizer* sizer);
+	void CreateAudioGroup(wxSizer* sizer);
+	void CreateTitleGroup(wxSizer* sizer);
 	bool SetValues();
 	/** Returns list of pre commands */
 	wxArrayString GetPreCommands();
@@ -42,6 +50,7 @@ private:
 	void OnAddBt(wxCommandEvent& event);
 	void OnRemoveBt(wxCommandEvent& event);
 	void OnPropBt(wxCommandEvent& event);
+	void OnChangeAudio(wxCommandEvent& event);
 	void OnDoNotTranscodeCheck(wxCommandEvent& event);
 	void UpdateDoNotTranscodeCheck();
 	DECLARE_EVENT_TABLE()
