@@ -3,7 +3,7 @@
 // Purpose:     Titleset Manager
 // Author:      Alex Thuering
 // Created:	27.01.2003
-// RCS-ID:      $Id: TitlesetManager.cpp,v 1.45 2010/09/05 10:56:08 ntalex Exp $
+// RCS-ID:      $Id: TitlesetManager.cpp,v 1.47 2011/02/20 18:56:27 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -456,6 +456,8 @@ bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi)
 	if (!createTitle && pgc != NULL)
 		cStr = wxT("0:00") + wxString(cStr.length() ? wxT(",") + cStr : wxT(""));
 	vob->SetChapters(cStr);
+	
+	// add vob
 	if (createTitle || pgc == NULL) {
 		pgc = new Pgc;
 		pgc->SetPostCommands(_T("call vmgm menu 1;"));
@@ -476,6 +478,15 @@ bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi)
 	} else
 		pgc->GetVobs().Add(vob);
 	
+	// set aspect ratio if it isn't set
+	if (pgcs.GetVideo().GetAspect() == arAUTO) {
+		float aspect = vob->GetVideoStream()->GetSourceAspectRatio();
+		if (aspect > 0)
+			pgcs.GetVideo().SetAspect(aspect < 1.4 ? ar4_3 : ar16_9);
+		else
+			pgcs.GetVideo().SetAspect(m_dvd->GetAspectRatio());
+	}
+	
 	UpdateItems();
 	SendDvdChangedEvent();
 	
@@ -543,7 +554,9 @@ void TitlesetManager::AddImage(const wxString& fname, bool createTitle) {
 
 	Slideshow* slideshow = pgc->GetSlideshow();
 	if (slideshow == NULL) {
-		slideshow = new Slideshow(m_dvd->GetVideoFormat(), m_dvd->GetAspectRatio());
+		if (pgcs.GetVideo().GetAspect() == arAUTO)
+			pgcs.GetVideo().SetAspect(m_dvd->GetAspectRatio());
+		slideshow = new Slideshow(m_dvd->GetVideoFormat(), pgcs.GetVideo().GetAspect());
 		Vob* vob = new Vob(slideshow);
 		pgc->GetVobs().Add(vob);
 	}
@@ -873,6 +886,7 @@ void TitlesetManager::OnProps(wxCommandEvent& WXUNUSED(event)) {
 			m_dvd->UpdateButtonImageFor(tsi, pgci);
 			SendMenuSelectedEvent(); // update MenuEditor
 			SendDvdChangedEvent();
+			UpdateItems();
 		}
 	}
 	m_pointed = -1;
