@@ -3,7 +3,7 @@
 // Purpose:     Media Encoder Interface
 // Author:      Alex Thuering
 // Created:     24.02.2007
-// RCS-ID:      $Id: mediaenc.h,v 1.6 2010/05/03 06:06:48 ntalex Exp $
+// RCS-ID:      $Id: mediaenc.h,v 1.7 2011/02/20 18:51:35 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -47,6 +47,8 @@ enum AspectRatio {
 
 /** Returns frame size of given video format */
 wxSize GetFrameSize(VideoFormat format);
+/** Returns frame aspect ratio */
+double GetFrameAspectRatio(VideoFormat format, AspectRatio aspect);
 /** Returns true if video format is NTSC */
 bool isNTSC(VideoFormat format);
 
