@@ -3,7 +3,7 @@
 // Purpose:     FFMPEG Media Transcoder
 // Author:      Alex Thuering
 // Created:     26.04.2008
-// RCS-ID:      $Id: mediatrc_ffmpeg.cpp,v 1.27 2010/10/10 15:55:37 ntalex Exp $
+// RCS-ID:      $Id: mediatrc_ffmpeg.cpp,v 1.30 2011/03/10 20:08:22 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -628,8 +628,14 @@ bool wxFfmpegMediaTranscoder::AddVideoStream(AVFormatContext *oc,
 		wxSize frameSize = GetFrameSize(videoFormat);
 		frame_width = frameSize.GetWidth() - frame_padright - frame_padleft;
 		frame_height = frameSize.GetHeight() - frame_padtop - frame_padbottom;
-		frame_rate.num = isNTSC(videoFormat) ? 30000 : 25;
-		frame_rate.den = isNTSC(videoFormat) ? 1001 : 1;
+		if (!isNTSC(videoFormat)) {
+			frame_rate.num = 25;
+			frame_rate.den = 1;
+		} else if ((frame_rate.num != 24000 || frame_rate.num != 30000) && frame_rate.den != 1001) {
+			double fr = frame_rate.num > 0 && frame_rate.den > 0 ? ((double) frame_rate.num)/frame_rate.den : 29.97;
+			frame_rate.num = fr > 26 ? 30000 : 24000;
+			frame_rate.den = 1001;
+		}
 	}
 
 	st = av_new_stream(oc, oc->nb_streams);
@@ -687,7 +693,7 @@ bool wxFfmpegMediaTranscoder::AddVideoStream(AVFormatContext *oc,
 
 		video_enc->width = frame_width + frame_padright + frame_padleft;
 		video_enc->height = frame_height + frame_padtop + frame_padbottom;
-		video_enc->sample_aspect_ratio = av_d2q(frame_aspect_ratio*video_enc->height/video_enc->width, 255);
+		video_enc->sample_aspect_ratio = av_d2q(frame_aspect_ratio*frame_height/frame_width, 255);
 		video_enc->pix_fmt = frame_pix_fmt;
 		st->sample_aspect_ratio = video_enc->sample_aspect_ratio; 
 
@@ -873,7 +879,7 @@ bool wxFfmpegMediaTranscoder::AddSubtitleStream(AVFormatContext *oc,
 	}
 #endif
 
-	return false;
+	return true;
 }
 
 int read_file(const char *filename, char **bufptr, size_t *size)
@@ -926,6 +932,16 @@ void wxFfmpegMediaTranscoder::SetFirstField(FirstField firstField) {
 	top_field_first = firstField;
 }
 
+/** Sets start time */
+void wxFfmpegMediaTranscoder::SetStartTime(double startTime) {
+	start_time = (int64_t) (startTime * AV_TIME_BASE);
+}
+
+/** Sets recording time */
+void wxFfmpegMediaTranscoder::SetRecordingTime(double recordingTime) {
+	recording_time = recordingTime >= 0 ? (int64_t) (recordingTime * AV_TIME_BASE) : INT64_MAX;
+}
+
 /** Starts transcoding. */
 bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 	int ret = 0, i, j, k, n, nb_istreams = 0, nb_ostreams = 0;
