@@ -3,7 +3,7 @@
 // Purpose:     Pipe execute
 // Author:      Alex Thuering
 // Created:     23.08.2004
-// RCS-ID:      $Id: PipeExecute.cpp,v 1.5 2010/04/05 16:17:04 ntalex Exp $
+// RCS-ID:      $Id: PipeExecute.cpp,v 1.8 2010/09/13 19:48:43 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -268,7 +268,10 @@ bool wxPipeExecute::Execute(wxString command,
     wxString cmd = cmds[i];
 #ifdef __WXMSW__
     if (cmd.BeforeFirst(wxT(' ')).Find(wxT(':')) == -1)
-      cmd = wxGetAppPath() + wxString(wxFILE_SEP_PATH) + cmd;
+      cmd = wxGetAppPath() + cmd;
+#elif __WXMAC__
+    if (cmd.length() > 0 && cmd[0] != wxT('/'))
+      cmd = wxGetAppPath() + cmd;
 #endif
     pid[i] = wxExecute(cmd, wxEXEC_ASYNC, proc[i]);
     if (pid[i]<=0)
@@ -282,17 +285,16 @@ bool wxPipeExecute::Execute(wxString command,
     {
 	  for (int i=0; i<cnt; i++)
 		wxProcess::Kill(pid[i], wxSIGTERM);
+	  wxMilliSleep(200);
+	  for (int i=0; i<cnt; i++)
+	  	wxProcess::Kill(pid[i], wxSIGKILL);
       break;
     }
 	hasInput = false;
 	for (int i=0; i<cnt; i++)
 	  hasInput = hasInput || proc[i]->HasInput();
 	if (!hasInput)
-#if wxCHECK_VERSION(2,6,0)
 	  wxMilliSleep(50);
-#else
-      wxUsleep(50);
-#endif
   }
   bool res = proc[cnt-1]->GetExitCode() == 0;
   for (int i=0; i<cnt; i++)
