@@ -3,7 +3,7 @@
 // Purpose:     Properties dialog (base class for all properties dialogs)
 // Author:      Alex Thuering
 // Created:     18.11.2003
-// RCS-ID:      $Id: PropDlg.cpp,v 1.24 2010/10/10 18:37:52 ntalex Exp $
+// RCS-ID:      $Id: PropDlg.cpp,v 1.25 2011/02/13 17:56:31 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -703,7 +703,7 @@ void wxPropDlg::OnSelectColour(wxCommandEvent& event)
 }
 
 void wxPropDlg::AddFileProp(wxSizer* sizer, const wxString& label,
-  const wxString& value, int dlgStyle, wxString caption, wxString wildcard)
+  const wxString& value, int dlgStyle, wxString caption, wxString wildcard, int id)
 {
   if (!sizer) // only update value
   {
@@ -716,7 +716,7 @@ void wxPropDlg::AddFileProp(wxSizer* sizer, const wxString& label,
   if (label.length())
 	sizer->Add(new wxStaticText(propWindow, -1, label), 0, wxALIGN_CENTER_VERTICAL);
   wxBoxSizer* sizer2 = new wxBoxSizer(wxHORIZONTAL);
-  wxTextCtrl* ctrl = new wxTextCtrl(propWindow, -1, value);
+  wxTextCtrl* ctrl = new wxTextCtrl(propWindow, id, value);
   sizer2->Add(ctrl, 1, wxEXPAND);
   wxButton* button = new wxButton(propWindow, SELECT_FILE_BT_ID, caption);
   int h = ctrl->GetSize().GetHeight() > 24 ? ctrl->GetSize().GetHeight() : 24;
