@@ -3,7 +3,7 @@
 // Purpose:     wxThumbnailFactory class
 // Author:      Alex Thuering
 // Created:		15.02.2003
-// RCS-ID:      $Id: ThumbnailFactory.cpp,v 1.17 2010/09/05 14:35:58 ntalex Exp $
+// RCS-ID:      $Id: ThumbnailFactory.cpp,v 1.18 2011/02/23 20:24:10 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -269,13 +269,15 @@ wxImage wxThumbnailFactory::LoadThumbnail(ThumbInfo& info, ThumbType type) {
 	if (pixbuf) {
 		if (gnome_thumbnail_is_valid(pixbuf, info.uri, info.mtime)) {
 			wxLogNull log;
-			img.LoadFile(thumbPath); // img = pixbuf2image(pixbuf); failed on some systems
+			if (wxFileExists(thumbPath))
+				img.LoadFile(thumbPath); // img = pixbuf2image(pixbuf); failed on some systems
 		}
 		g_object_unref(pixbuf);
 	}
 #else
 	wxLogNull log;
-	img.LoadFile(thumbPath);
+	if (wxFileExists(thumbPath))
+		img.LoadFile(thumbPath);
 #endif
 	return img;
 }
@@ -333,11 +335,14 @@ wxImage wxThumbnailFactory::GenerateThumbnail(ThumbInfo& info, bool save) {
 		info.pixbuf = gnome_thumbnail_factory_generate_thumbnail(thumbnail_factory, info.uri, info.mime_type);
 		if (info.pixbuf) {
 			gnome_thumbnail_factory_save_thumbnail(thumbnail_factory, info.pixbuf, info.uri, info.mtime);
-			img.LoadFile(thumbPath);
+			wxLogNull log;
+			if (wxFileExists(thumbPath))
+				img.LoadFile(thumbPath);
 		} else
 			gnome_thumbnail_factory_create_failed_thumbnail(thumbnail_factory, info.uri, info.mtime);
 		return img;
-#elif defined(WX_SVG) || defined(decoder)
+#elif defined(WX_SVG)
+		wxLogNull log;
 		wxFfmpegMediaDecoder decoder;
 		if (decoder.Load(info.filename) && decoder.BeginDecode(96, 80)) {
 			if (decoder.GetDuration() > 0) {
@@ -353,6 +358,13 @@ wxImage wxThumbnailFactory::GenerateThumbnail(ThumbInfo& info, bool save) {
 				for (int i = 0; i < 15; i++)
 					img = decoder.GetNextFrame();
 			}
+			if (img.Ok() && decoder.GetFrameAspectRatio() > 0 
+					&& abs(decoder.GetFrameAspectRatio()*100 - img.GetWidth()*100/img.GetHeight()) >= 5) {
+				if (lround(img.GetWidth() / decoder.GetFrameAspectRatio()) > img.GetHeight())
+					img.Rescale(img.GetWidth(), lround(img.GetWidth() / decoder.GetFrameAspectRatio()));
+				else
+					img.Rescale(lround(img.GetHeight() * decoder.GetFrameAspectRatio()), img.GetHeight());
+			}
 		}
 		if (img.Ok()) {
 			unsigned char* data = img.GetData();
