@@ -3,7 +3,7 @@
 // Purpose:     wxThumbnails class
 // Author:      Alex Thuering
 // Created:		2.02.2003
-// RCS-ID:      $Id: Thumbnails.cpp,v 1.30 2010/06/29 18:41:10 ntalex Exp $
+// RCS-ID:      $Id: Thumbnails.cpp,v 1.31 2011/02/23 20:24:10 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -220,7 +220,7 @@ void wxThumbnails::SetSelected(int value) {
 }
 
 wxString wxThumbnails::GetAudioVideoExtWildcard() {
-	return wxT("*.asf;*.avi;*.dv;*.flv;*.m2p;*.mkv;*.mov;*.mp4;*.mpeg;*.mpg;*.ogg;*.ogm;*.vob;*.webm;*.wmv;");
+	return wxT("*.asf;*.avi;*.dv;*.flv;*.m2p;*.mkv;*.mov;*.mp4;*.mpeg;*.mpg;*.ogg;*.ogm;*.vob;*.vro;*.webm;*.wmv;");
 }
 
 wxString wxThumbnails::GetVideoExtWildcard() {
